// RootDetector.cpp : Defines the entry point for the console application.
//

//#include "stdafx.h"
#include "RootDetector.h"
#include "RootDetectorInternal.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace std;
#define pi 3.1415926
#define BUFFER_LENGTH   80
#define stackSize 16777216
int stack[stackSize];
int stackPointer;

void Otsu(ImgGray& img, unsigned char* threshold_val)
{
	std::vector<int> histogram_bin;
	std::vector<double> histogram_normalize;
	histogram_bin.clear();
	histogram_normalize.clear();
	HistogramGray(img, 256, &histogram_bin);
	int hist_sum = 0;
	for(int nn=0; nn<256; nn++)
	{
		hist_sum = hist_sum + histogram_bin[nn];
	}
	for(nn=0; nn<256; nn++)
	{
		histogram_normalize.push_back(double(histogram_bin[nn])/double(hist_sum));
	}
	int min_level, max_level;
	min_level = 255;
	max_level = 0;
	unsigned char* pt = img.Begin();
	for (nn=0 ; nn<img.Width()*img.Height(); nn++) 
	{
		int xx = *pt++;
		if(xx > max_level)
		{
			max_level = xx;
		}
		if(xx < min_level)
		{
			min_level = xx;
		}
	}
	min_level=min_level;
	max_level =max_level;
	float ut = 0.0;
	for(nn=0; nn<256; nn++)
	{
		ut = ut + histogram_normalize[nn]*nn;
	}
	float dett = 0.0;
	for(nn=0; nn<256; nn++)
	{
		dett = dett + histogram_normalize[nn]*pow((nn-ut),2);
	}
	float w0 = 0.0;
	float w1 = 0.0;
	float mx = 0.0;
	float u1, na, detb;
	float u0 = 0.0;
	for(nn=min_level; nn<=max_level-1; nn++)
	{
		w0 = w0 + histogram_normalize[nn];
		w1 = 1.0 - w0;
		u0 = 0.0;
		for(int mm=0; mm<=nn; mm++)
		{
			u0 = u0 + histogram_normalize[mm]*mm/w0;
		}
		u1 = (ut-w0*u0)/w1;
		detb = w0*w1*pow((u0-u1), 2);
		na = detb/dett;
		if( na > mx)
		{
			mx = na;
			*(threshold_val) = nn-1;
		}
	}
}


void Sobel(const ImgGray& img, ImgGray* out)
{
     int sobel_kernel_x[3][3];
     sobel_kernel_x[0][0] = -1 ;
     sobel_kernel_x[0][1] = 0 ;
     sobel_kernel_x[0][2] = 1 ;
     sobel_kernel_x[1][0] = -2 ;
     sobel_kernel_x[1][1] = 0 ;
     sobel_kernel_x[1][2] = 2 ;
     sobel_kernel_x[2][0] = -1 ;
     sobel_kernel_x[2][1] = 0 ;
     sobel_kernel_x[2][2] = 1 ;

     int sobel_kernel_y[3][3];
     sobel_kernel_y[0][0] = 1 ;
     sobel_kernel_y[0][1] = 2 ;
     sobel_kernel_y[0][2] = 1 ;
     sobel_kernel_y[1][0] = 0 ;
     sobel_kernel_y[1][1] = 0 ;
     sobel_kernel_y[1][2] = 0 ;
     sobel_kernel_y[2][0] = -1 ;
     sobel_kernel_y[2][1] = -2 ;
     sobel_kernel_y[2][2] = -1 ;
  
     int w = img.Width();
     int h = img.Height();
     out->Reset(w, h);
     *out = img;

     int Gx, Gy, Gs;
     Gx = 0;
     Gy = 0;
     Gs = 0;
     for ( int i = 1; i < w-1; i++ )
	 {
	      for ( int j = 1; j < h-1; j++ )
		  {
		      Gx = 0;
		      Gy = 0;
		      Gs = 0;
		      Gx = Gx + sobel_kernel_x[0][0] * (*(img).Begin(i-1,j-1))
			          + sobel_kernel_x[0][1] * (*(img).Begin(i,j-1))
				      + sobel_kernel_x[0][2] * (*(img).Begin(i+1,j-1))
				      + sobel_kernel_x[1][0] * (*(img).Begin(i-1,j))
				      + sobel_kernel_x[1][1] * (*(img).Begin(i,j))
				      + sobel_kernel_x[1][2] * (*(img).Begin(i+1,j))
				      + sobel_kernel_x[2][0] * (*(img).Begin(i-1,j+1))
				      + sobel_kernel_x[2][1] * (*(img).Begin(i,j+1))
				      + sobel_kernel_x[2][2] * (*(img).Begin(i+1,j+1));
		      if( Gx < 0 )
			  {
			      Gx = -1*Gx;
			  }

		      Gy = Gy + sobel_kernel_y[0][0] * (*(img).Begin(i-1,j-1))
			          + sobel_kernel_y[0][1] * (*(img).Begin(i,j-1))
				      + sobel_kernel_y[0][2] * (*(img).Begin(i+1,j-1))
				      + sobel_kernel_y[1][0] * (*(img).Begin(i-1,j))
				      + sobel_kernel_y[1][1] * (*(img).Begin(i,j))
				      + sobel_kernel_y[1][2] * (*(img).Begin(i+1,j))
				      + sobel_kernel_y[2][0] * (*(img).Begin(i-1,j+1))
				      + sobel_kernel_y[2][1] * (*(img).Begin(i,j+1))
				      + sobel_kernel_y[2][2] * (*(img).Begin(i+1,j+1));
		      if( Gy < 0 )
			  {
			      Gy = -1*Gy;
			  }
		  
		      Gs = Gx + Gy;
		      if ( Gs > 255 )
			  {
			      Gs = 255;
			  }
              *(out)->Begin(i,j) = Gs;
		  }
	 }
}


bool pop(int &x, int &y, int w) 
{ 
    if(stackPointer > 0) 
    { 
        int p = stack[stackPointer]; 
        y = p / w; 
        x = p % w; 
        stackPointer--; 
        return 1; 
    }     
    else 
    { 
        return 0; 
    }    
}    

bool push(int x, int y, int w) 
{ 
    if(stackPointer < stackSize - 1) 
    { 
        stackPointer++; 
        stack[stackPointer] = y * w + x; 
        return 1; 
    }     
    else 
    { 
        return 0; 
    }    
}     

void emptyStack(int x, int y, int w) 
{  
    while(pop(x, y, w)); 
}

void floodFill4Stack(ImgGray& img, int x, int y, int newColor, int oldColor, int half_width, std::vector<LineParameter>& line_para, std::vector<int>* flood_points_x, std::vector<int>* flood_points_y)
{
	int ImgWidth = img.Width();
    int ImgHeight = img.Height();

    double local_dist_val;
    local_dist_val = 0.0;
	double local_down, local_up, local_min;

	if(newColor == oldColor) return; //avoid infinite loop
    emptyStack(x, y, ImgWidth);
    
    if(!push(x, y, ImgWidth)) return; 
    while(pop(x, y, ImgWidth))
    {
        img(x,y) = newColor;
        flood_points_x->push_back(x);
		flood_points_y->push_back(y);
        local_min = 9999.0;
		for(int kkk=0; kkk<line_para.size(); kkk++)
		{
            local_down = pow(line_para[kkk].a, 2) + pow(line_para[kkk].b, 2);
		    local_up = abs(line_para[kkk].a*y + line_para[kkk].b*(x+1) - line_para[kkk].c);
		    local_dist_val = local_up/sqrt(local_down);
			if( local_dist_val < local_min )
			{
				local_min = local_dist_val;
			}
		}
        if((x + 1 < ImgWidth) && (img(x+1,y) >= oldColor) && (local_min <= half_width) )
        {          
            if(!push(x + 1, y, ImgWidth)) return;           
        } 

		local_min = 9999.0;
		for(kkk=0; kkk<line_para.size(); kkk++)
		{
            local_down = pow(line_para[kkk].a, 2) + pow(line_para[kkk].b, 2);
		    local_up = abs(line_para[kkk].a*y + line_para[kkk].b*(x-1) - line_para[kkk].c);
		    local_dist_val = local_up/sqrt(local_down);
			if( local_dist_val < local_min )
			{
				local_min = local_dist_val;
			}
		}
        if((x - 1 >= 0) && (img(x-1,y) >= oldColor) && (local_min <= half_width))
        {
            if(!push(x - 1, y, ImgWidth)) return;           
        }

		local_min = 9999.0;
		for(kkk=0; kkk<line_para.size(); kkk++)
		{
            local_down = pow(line_para[kkk].a, 2) + pow(line_para[kkk].b, 2);
		    local_up = abs(line_para[kkk].a*(y+1) + line_para[kkk].b*x - line_para[kkk].c);
		    local_dist_val = local_up/sqrt(local_down);
			if( local_dist_val < local_min )
			{
				local_min = local_dist_val;
			}
		}
        if((y + 1 < ImgHeight) && (img(x,y+1) >= oldColor) && (local_min <= half_width))
        {
            if(!push(x, y + 1, ImgWidth)) return;           
        } 
		
		local_min = 9999.0;
		for(kkk=0; kkk<line_para.size(); kkk++)
		{
            local_down = pow(line_para[kkk].a, 2) + pow(line_para[kkk].b, 2);
		    local_up = abs(line_para[kkk].a*(y-1) + line_para[kkk].b*x - line_para[kkk].c);
		    local_dist_val = local_up/sqrt(local_down);
			if( local_dist_val < local_min )
			{
				local_min = local_dist_val;
			}
		}
        if((y - 1 >= 0) && (img(x,y-1) >= oldColor) && (local_min <= half_width))
        {
            if(!push(x, y - 1, ImgWidth)) return;           
        }    
    }     
}

void floodFill4Binary(ImgGray& img, int x, int y, int newColor, int oldColor, int half_width, std::vector<double>& line_para, std::vector<int>* flood_points_x, std::vector<int>* flood_points_y)
{
	int ImgWidth = img.Width();
    int ImgHeight = img.Height();

    double local_dist_val;
    local_dist_val = 0.0;

	if(newColor == oldColor) return; //avoid infinite loop
    emptyStack(x, y, ImgWidth);
      
    if(!push(x, y, ImgWidth)) return; 
    while(pop(x, y, ImgWidth))
    {
        img(x,y) = newColor;
		flood_points_x->push_back(x);
		flood_points_y->push_back(y);

        double local_down = pow(line_para[0], 2) + pow(line_para[1], 2);
		double local_up = abs(line_para[0]*y + line_para[1]*(x+1) - line_para[2]);
		local_dist_val = local_up/sqrt(local_down);
		if( (x + 1 < ImgWidth) && (img(x+1,y) == oldColor) && (local_dist_val <= half_width) )
        {          
            if(!push(x + 1, y, ImgWidth)) return;           
        } 
        
		local_up = abs(line_para[0]*y + line_para[1]*(x-1) - line_para[2]);
		local_dist_val = local_up/sqrt(local_down);
        if( (x - 1 >= 0) && (img(x-1,y) == oldColor) && (local_dist_val <= half_width))
		{
            if(!push(x - 1, y, ImgWidth)) return;           
        }

		local_up = abs(line_para[0]*(y+1) + line_para[1]*x - line_para[2]);
		local_dist_val = local_up/sqrt(local_down);
        if( (y + 1 < ImgHeight) && (img(x,y+1) == oldColor) && (local_dist_val <= half_width))
		{
            if(!push(x, y + 1, ImgWidth)) return;           
        } 
		
		local_up = abs(line_para[0]*(y-1) + line_para[1]*x - line_para[2]);
		local_dist_val = local_up/sqrt(local_down);
        if( (y - 1 >= 0) && (img(x,y-1) == oldColor) && (local_dist_val <= half_width))
		{
            if(!push(x, y - 1, ImgWidth)) return;           
        }    
    } 
       
}


void FindSkelton(const ImgGray& root_half, std::vector<int>* SkeletonX, std::vector<int>* SkeletonY)
{
	int kernel_x[10], kernel_y[10];
    kernel_x[0]=0; kernel_x[1]=0;  kernel_x[2]=1;  kernel_x[3]=1;  kernel_x[4]=1;
    kernel_x[5]=0; kernel_x[6]=-1; kernel_x[7]=-1; kernel_x[8]=-1; kernel_x[9]=0;
	kernel_y[0]=0; kernel_y[1]=-1; kernel_y[2]=-1; kernel_y[3]=0;  kernel_y[4]=1;
    kernel_y[5]=1; kernel_y[6]=1;  kernel_y[7]=0;  kernel_y[8]=-1; kernel_y[9]=-1;
	
	ImgGray temp_binary; 
	int ImgSizeXHalf, ImgSizeYHalf;
    ImgSizeXHalf = root_half.Width();
    ImgSizeYHalf = root_half.Height();

	temp_binary.Reset(ImgSizeXHalf, ImgSizeYHalf);
	Set(&temp_binary, 0);

	for( int cc = 0; cc < ImgSizeXHalf; cc++)
	{
		for( int rr = 0; rr < ImgSizeYHalf; rr++)
		{
			if( root_half(cc,rr) == 255 )
			{
				temp_binary(cc,rr) = 1;
			}
		}
	}

	while (1)
	{   
		ImgGray C; 
		C.Reset(ImgSizeXHalf, ImgSizeYHalf);
		Erode3x3(temp_binary, &C);
		Subtract(temp_binary, C, &C);
			    
		ImgGray D; 
		D.Reset(ImgSizeXHalf, ImgSizeYHalf);
		Set(&D, 0);
		for( cc = 1; cc < (ImgSizeXHalf-1); cc++)
		{
			for( int rr = 1; rr < (ImgSizeYHalf-1); rr++)
			{
				int N = 0;
				if( C(cc, rr) > 0 )
				{
					for ( int s=1; s<9; s++)
					{
						N=N+temp_binary(cc+kernel_y[s], rr+kernel_x[s]);
					}
				}
				if( (N>=2) && (N<=6) )
				{
					int SS = 0;
					for ( int s=1; s<9; s++)
					{
						if ( (temp_binary(cc+kernel_y[s], rr+kernel_x[s]) == 0) & ( temp_binary(cc+kernel_y[s+1], rr+kernel_x[s+1]) > 0) )
						{
							SS = SS + 1;
						}
					}
					if (SS==1)
					{
						if ( temp_binary(cc+kernel_y[1], rr+kernel_x[1])*temp_binary(cc+kernel_y[3], rr+kernel_x[3])*temp_binary(cc+kernel_y[5], rr+kernel_x[5]) == 0 )
						{
							if ( temp_binary(cc+kernel_y[3], rr+kernel_x[3])*temp_binary(cc+kernel_y[5], rr+kernel_x[5])*temp_binary(cc+kernel_y[7], rr+kernel_x[7]) == 0 )
							{
								D(cc, rr) = 1;
							}
						}
					}
				}
			}
		}
		Xor(temp_binary, D, &temp_binary);
		C.Reset(ImgSizeXHalf, ImgSizeYHalf);
		Erode3x3(temp_binary, &C);
		Subtract(temp_binary, C, &C);
		
		ImgGray DD; 
		DD.Reset(ImgSizeXHalf, ImgSizeYHalf);
		Set(&DD, 0);
		for( cc = 1; cc < (ImgSizeXHalf-1); cc++)
		{
			for( int  rr = 1; rr < (ImgSizeYHalf-1); rr++)
			{
				int N = 0;
                if( C(cc, rr) > 0 )
				{
					for ( int s=1; s<9; s++)
					{
						N=N+temp_binary(cc+kernel_y[s], rr+kernel_x[s]);
					}
				}
				
				if( (N>=2) && (N<=6) )
				{
					int SS = 0;
					for ( int s=1; s<9; s++)
					{
						if ( (temp_binary(cc+kernel_y[s], rr+kernel_x[s])==0.0) & (temp_binary(cc+kernel_y[s+1], rr+kernel_x[s+1])>0) )
						{
							SS = SS + 1;
						}
					}

					if (SS==1)
					{
						if ( temp_binary(cc+kernel_y[1], rr+kernel_x[1])*temp_binary(cc+kernel_y[3], rr+kernel_x[3])*temp_binary(cc+kernel_y[7], rr+kernel_x[7]) == 0 )
						{
							if ( temp_binary(cc+kernel_y[1], rr+kernel_x[1])*temp_binary(cc+kernel_y[5], rr+kernel_x[5])*temp_binary(cc+kernel_y[7], rr+kernel_x[7]) == 0 )
							{
								DD(cc, rr) = 1;
							}
						}
					}
				}
			}
		}
		Xor(temp_binary, DD, &temp_binary);
				
		Or(D, DD, &D);
		ImgGray DDD; 
		DDD.Reset(ImgSizeXHalf, ImgSizeYHalf);
		Set(&DDD, 0);
		if ( IsIdentical(D, DDD)==1 )
		{
			break;
		}
	}
	

	    for( int rr = 1; rr < (ImgSizeYHalf-1); rr++)
		{
		    for( cc = 1; cc < (ImgSizeXHalf-1); cc++)
			{
			    if ( temp_binary(cc,rr) > 0 )
				{
				    int white_no = 0;
				    for (int s = -1; s <=1; s++)
					{
					    for (int t = -1; t <=1; t++)
						{
						    if(temp_binary(cc+s,rr+t))
							{
							    white_no = white_no + 1;
							}
						}
					}
				    if( white_no == 1 )
					{
					    temp_binary(cc,rr) = 0;
					}
				    else
					{
				        SkeletonX->push_back(cc);
				        SkeletonY->push_back(rr);
					}
				}
			}
		}
}

void FindLengthPath(ImgGray& RootArea, double AreaOrient, std::vector<SeedPoint>* centerline_points)
{
	ImgGray root_half;
	root_half = RootArea;

	int ImgSizeXX = root_half.Width();
	int ImgSizeYY = root_half.Height();
	ImgGray root_transpose;
	std::vector<int> SkeletonPointX;
	std::vector<int> SkeletonPointY;
	SkeletonPointX.clear();
	SkeletonPointY.clear();

	if( (AreaOrient >= 135) || (AreaOrient <= 45))
	{
		std::vector<int> trans_skeleton_x;
		std::vector<int> trans_skeleton_y;
		trans_skeleton_x.clear();
		trans_skeleton_y.clear();
		std::vector<int> trans_centeraxis_x;
		std::vector<int> trans_centeraxis_y;
		trans_centeraxis_x.clear();
		trans_centeraxis_y.clear();
		root_transpose.Reset(ImgSizeYY, ImgSizeXX);
        unsigned char *ptr4 = root_half.Begin();
		for (int y=0 ; y<root_half.Height() ; y++)
		{
			unsigned char *ptr5 = root_transpose.Begin(y, 0);
			for (int x=0 ; x<root_half.Width() ; x++)
			{
				*ptr5 = *ptr4++;
				ptr5 += root_transpose.Width();
			}
		}
		FindSkelton(root_transpose, &trans_skeleton_x, &trans_skeleton_y);
		SkeletonPointX = trans_skeleton_y;
		SkeletonPointY = trans_skeleton_x;
	}
	else
	{
		root_transpose.Reset(ImgSizeXX, ImgSizeYY);	
		root_transpose = root_half;
		FindSkelton(root_transpose, &SkeletonPointX, &SkeletonPointY);
	}

	ImgFloat matrix;
    matrix.Reset(SkeletonPointX.size(), SkeletonPointX.size());
	Set(&matrix,0);
	std::vector<int> farthestPreviousHop;
	std::vector<int> farthestNextHop;
    double totalCost = 0.0;
    double points_distance = 0.0;
	if( SkeletonPointX.size() > 0 )
	{
		double R =1.5;
		for ( int iii = 0; iii<SkeletonPointX.size(); iii++)
		{
			for ( int jjj = 0; jjj<SkeletonPointX.size(); jjj++)
			{
				if ( iii == jjj )
				{
					points_distance = 0.0;
				}
				else
				{
					points_distance = sqrt( pow((SkeletonPointX[iii]-SkeletonPointX[jjj]), 2) + pow((SkeletonPointY[iii]-SkeletonPointY[jjj]), 2) );
					if( points_distance <= R )
					{
						matrix(iii, jjj) = points_distance;
					}
					else
					{
						matrix(iii, jjj) = 99999.0;
					}
				}
			}
		}

		std::vector<int> activeNodes;
		for ( iii = 0; iii<SkeletonPointX.size(); iii++)
		{
			farthestPreviousHop.push_back(iii);
			farthestNextHop.push_back(iii);
		}
        
		int SkeletonPointsNo = SkeletonPointX.size();
		int start_no = 0;
        int destination_no =  SkeletonPointX.size()-1;

		std::vector<int> visited;
		std::vector<double> distance;
		std::vector<int> parent;
		std::vector<double> temp_distance;
		std::vector<double> temp_distance_backup;
		visited.clear();
		distance.clear();
		parent.clear();
		temp_distance.clear();
		temp_distance_backup.clear();
		int lowest_id = 0;

		for(iii=0; iii<SkeletonPointsNo; iii++)
		{
			visited.push_back(0);
			parent.push_back(0);
			distance.push_back(99999.0);
		}

		distance[start_no] = 0.0;
		for (iii = 0; iii<(SkeletonPointsNo-1); iii++)
		{
			temp_distance.clear();
			temp_distance_backup.clear();
			for (int hhh = 0; hhh<SkeletonPointsNo; hhh++)
			{
				if( visited[hhh] == 0 )
				{
					temp_distance.push_back(distance[hhh]);
				}
				else
				{
					temp_distance.push_back(99999.0);
				}
			}
			temp_distance_backup = temp_distance;
			sort(temp_distance.begin(), temp_distance.end());
			for(int gg=0; gg<temp_distance.size(); gg++)
			{
				if( temp_distance[0] == temp_distance_backup[gg])
				{
					lowest_id = gg+1;
					break;
				}	
			}
			
			visited[lowest_id-1] = 1;
			for(int v = 0; v<SkeletonPointsNo; v++)
			{
				if( (matrix(lowest_id-1, v)<99999.0) && (distance[lowest_id-1]<99999.0) )
				{   
					if( (matrix(lowest_id-1, v) + distance[lowest_id-1]) < distance[v] )
					{
						distance[v]	= distance[lowest_id-1] + matrix(lowest_id-1, v);
						parent[v] = lowest_id-1;
					}
				}
			}
		}
		
		std::vector<int> path;
		path.clear();
		if( parent[SkeletonPointX.size()-1] != 0 )
		{
			int t = SkeletonPointX.size()-1;
			path.push_back(SkeletonPointX.size()-1);
			while(t != start_no)
			{
				int p = parent[t];
				std::vector<int>::iterator it = path.begin();
				path.insert(it, 1, p);
				
				if( matrix(t,farthestPreviousHop[t]) < matrix(t,p) )
				{
					farthestPreviousHop[t] = p;
				}
				if( matrix(p,farthestNextHop[p]) < matrix(p,t) )
				{
					farthestNextHop[p] = t;
				}
				t = p;
			}
		}
		
		for(int xxx=0; xxx<path.size(); xxx++)
		{
			SeedPoint tmp;
			tmp.x = SkeletonPointX[path[xxx]];
			tmp.y = SkeletonPointY[path[xxx]];
			centerline_points->push_back(tmp);
		}

    }
}

void FindDiameter(ImgGray& RootArea, std::vector<SeedPoint>& centeraxis_points, double* root_diameter, int* salient)
{
	ImgGray root_half;
	root_half = RootArea;
	std::vector<double> centeraxis_theta;
	centeraxis_theta.clear();
	if ( centeraxis_points.size() < 10 )
	{
		double theta_sum;
		double theta_mean;
		int ver_slope, hor_slope, avg_slope;
		theta_sum = 0.0;
		theta_mean = 0.0;
		ver_slope = 0;
		hor_slope = 0;
        avg_slope = 0;
		int loc_middle = floor(centeraxis_points.size()/2);
		for ( int nnn = 0; nnn<centeraxis_points.size(); nnn++ )
		{
		    int coordinate_x, coordinate_y;
			double theta_slope;
			theta_slope = 0.0;
            coordinate_x = 0;
			coordinate_y = 0;
            coordinate_x = centeraxis_points[nnn].x - centeraxis_points[loc_middle].x;
			coordinate_y = centeraxis_points[nnn].y - centeraxis_points[loc_middle].y;
			if( (coordinate_x != 0) && (coordinate_y != 0) )
			{
				theta_slope = double(coordinate_y)/double(coordinate_x);
				theta_sum = theta_sum+theta_slope;
				avg_slope = avg_slope + 1;
			}
			if( (coordinate_x == 0) && (coordinate_y != 0) )
			{
				ver_slope = ver_slope + 1;
			}
			if( (coordinate_x != 0) && (coordinate_y == 0) )
			{
				hor_slope = hor_slope + 1;
			}
		}
		if( hor_slope >= floor(centeraxis_points.size()/2) )
		{
			theta_mean = 0.01;
		}
		else if( ver_slope >= floor(centeraxis_points.size()/2) )
		{
			theta_mean = 99999.0;
		}
		else
		{
			theta_mean = theta_sum/avg_slope;
		}
		for(nnn=0; nnn<centeraxis_points.size(); nnn++)
		{
			centeraxis_theta.push_back(theta_mean);
		}
	}

	if ( centeraxis_points.size() >= 10 )
	{
		for( int nnn=4; nnn < (centeraxis_points.size()-5); nnn++)
		{
			double theta_sum;
			double theta_mean;
			int ver_slope, hor_slope, avg_slope;
			theta_sum = 0.0;
			theta_mean = 0.0;
			ver_slope = 0;
			hor_slope = 0;
			avg_slope = 0;
			for(int mmm = -4; mmm<=5; mmm++)
			{
				int coordinate_x, coordinate_y;
				double theta_slope;
				theta_slope = 0.0;
                coordinate_x = 0;
				coordinate_y = 0;
				ver_slope = 0;
				hor_slope = 0;
                coordinate_x = centeraxis_points[nnn+mmm].x - centeraxis_points[nnn].x;
				coordinate_y = centeraxis_points[nnn+mmm].y - centeraxis_points[nnn].y;
				if( (coordinate_x != 0) && (coordinate_y != 0) )
				{
					theta_slope = double(coordinate_y)/double(coordinate_x);
					theta_sum = theta_sum+theta_slope;
					avg_slope = avg_slope + 1;
				}
				if( (coordinate_x == 0) && (coordinate_y != 0) )
				{
					ver_slope = ver_slope + 1;
				}
				if( (coordinate_x != 0) && (coordinate_y == 0) )
				{
					hor_slope = hor_slope + 1;
				}
			}
			if( hor_slope >= 4 )
			{
				theta_mean = 0.01;
			}
			else if( ver_slope >= 4 )
			{
				theta_mean = 99999.0;
			}
			else
			{
				theta_mean = theta_sum/avg_slope;
			}
            centeraxis_theta.push_back(theta_mean);
		}
		std::vector<double>::iterator it = centeraxis_theta.begin();
		double insert_val = centeraxis_theta[0];
        centeraxis_theta.insert(it, 4, insert_val);
		it = centeraxis_theta.end();
		insert_val = centeraxis_theta[centeraxis_theta.size()-1];
		centeraxis_theta.insert(it, 4, insert_val);
	}
    std::vector<double> centeraxis_perpendicular;
	centeraxis_perpendicular.clear();
	for(int nnn = 0; nnn<centeraxis_theta.size(); nnn++)
	{
		centeraxis_perpendicular.push_back(-1.0/centeraxis_theta[nnn]);
	}
	
	ImgGray root_border, temp_border;
	root_border.Reset(root_half.Width(), root_half.Height());
	temp_border.Reset(root_half.Width(), root_half.Height());
	Erode3x3(root_half, &temp_border);
    Subtract(root_half, temp_border, &root_border);

	std::vector<int> border_x;
	std::vector<int> border_y; 
	border_x.clear();
	border_y.clear();

	unsigned char *ptr = root_border.Begin();
	for (int ni=0 ; ni<root_half.Width()*root_half.Height(); ni++) 
	{
		if(*ptr++ == 255)
		{
			int rr = ni/(root_half.Width());
			int cc = ni%(root_half.Width());
			border_x.push_back(cc);
			border_y.push_back(rr);
		}
	}
	
	double temp_diameter;
    int temp_y, temp_yy;
	std::vector<int> diameter_val;
	std::vector<int> border_up_x;
	std::vector<int> border_up_y;
	std::vector<int> border_down_x;
	std::vector<int> border_down_y;
	border_up_x.clear();
	border_up_y.clear();
	border_down_x.clear();
	border_down_y.clear();
	diameter_val.clear();

	nnn = 0;
	double sum_width = 0.0;
	while( nnn<centeraxis_points.size() )
	{
		border_up_x.clear();
	    border_up_y.clear();
		border_down_x.clear();
		border_down_y.clear();
		for(int mmm = 0; mmm<border_x.size(); mmm++)
		{
			if( (centeraxis_points[nnn].x!=border_x[mmm]) && (centeraxis_points[nnn].y!=border_y[mmm]) )
			{
				temp_y = floor(centeraxis_perpendicular[nnn]*(border_x[mmm]-centeraxis_points[nnn].x))+centeraxis_points[nnn].y-border_y[mmm];
				if( abs(temp_y) <= 2 )
				{
					temp_yy = floor(centeraxis_theta[nnn]*(border_x[mmm]-centeraxis_points[nnn].x))+centeraxis_points[nnn].y-border_y[mmm];
					if( temp_yy > 0 )
					{
						border_up_x.push_back(border_x[mmm]);
						border_up_y.push_back(border_y[mmm]);
					}
					if( temp_yy < 0 )
					{
						border_down_x.push_back(border_x[mmm]);
						border_down_y.push_back(border_y[mmm]);
					}
				}
			}
		}

		if( (border_up_x.size()>0) && (border_down_x.size()>0) )
		{
			temp_diameter = sqrt(pow((border_down_x[0]-border_up_x[0]), 2)+pow((border_up_y[0]-border_down_y[0]), 2));
			diameter_val.push_back(temp_diameter);
			sum_width = sum_width + temp_diameter;
		}
		nnn = nnn + 1;
	}
	if( diameter_val.size() > 0 )
	{
		int median_loc = 0;
	    double median_val = 0.0;
        sort(diameter_val.begin(), diameter_val.end());
	    median_loc = floor(diameter_val.size()/2);
	    double mean_width = sum_width/diameter_val.size();
	    double swd, temp_sum;
	    temp_sum = 0.0;
	    for(int xxx=0; xxx<diameter_val.size(); xxx++)
		{
            temp_sum = temp_sum+pow((diameter_val[xxx]-mean_width),2);
		}
        temp_sum = temp_sum/(diameter_val.size()-1);
        swd = sqrt(temp_sum);
	    if(swd >= 0.5)
		{
		    *(salient) =1;
		}
    	else
		{
		    *(salient) =0;
		}
	    if( 2*median_loc != diameter_val.size() )
		{
		    median_val = diameter_val[median_loc];
		}
	    else
		{
		    median_val = (diameter_val[median_loc]+diameter_val[median_loc-1])/2;
		}
		if(median_val < 8.0)
		{
			median_val = 8.0;
		}
        *(root_diameter) = median_val;
	}
	else
	{
		*(root_diameter) = 8.0;
	}
	
	
}

void GreyFloodFill(RootElement& root_centerline, ImgGray& grey_img, double* cover_percent,  std::vector<SeedPoint>* cover_points, std::vector<SeedPoint>* not_cover_points, std::vector<int>* filled_points_x, std::vector<int>* filled_points_y)
{
	int iii = 0;
	ImgGray tmp;
    int ite = floor(root_centerline.val);
	int wte = floor(root_centerline.width/2); 
    double cover_ratio = 0.0; 

	while( (cover_ratio<0.45) && (iii<root_centerline.points_x.size()) )
	{ 
		cover_points->clear();
		not_cover_points->clear();
		filled_points_x->clear();
		filled_points_y->clear();
		tmp = grey_img;
		int start_points_x = root_centerline.points_x[iii];
		int start_points_y = root_centerline.points_y[iii];
		floodFill4Stack(tmp, start_points_x, start_points_y, 0, ite, wte, root_centerline.total_line_para, filled_points_x, filled_points_y);
		for(int jj = 0; jj < root_centerline.points_x.size(); jj++)
		{
			start_points_x = root_centerline.points_x[jj];
			start_points_y = root_centerline.points_y[jj];
			int meet_already = 0;
			for(int kk = 0; kk < filled_points_x->size(); kk++)
			{
				if( (start_points_x == filled_points_x->at(kk)) && (start_points_y == filled_points_y->at(kk)) )
				{
					SeedPoint ttmp;
					ttmp.x = start_points_x;
					ttmp.y = start_points_y;
					cover_points->push_back(ttmp);
					meet_already = 1;
					break;
				}
			}
			if( meet_already == 0)
			{
				SeedPoint ttmp;
				ttmp.x = start_points_x;
				ttmp.y = start_points_y;
				not_cover_points->push_back(ttmp);
			}
		}
        cover_ratio = double(cover_points->size())/double(root_centerline.points_x.size());
		iii++;
	}
	*(cover_percent) = cover_ratio;
}

void RegionClean(std::vector<int>& filled_point_x, std::vector<int>& filled_point_y, int img_width, int img_height, ImgGray* temp_binary_2, std::vector<SeedPoint>* filled_clean_points)
{
	ImgGray temp_binary;
	temp_binary.Reset(img_width, img_height);
	Set(&temp_binary, 0);
	for(int jj=0; jj<filled_point_x.size(); jj++)
	{
		temp_binary(filled_point_x[jj], filled_point_y[jj]) = 255;
	}
	ImgGray temp_binary_1;
    temp_binary_1.Reset(img_width, img_height);
	Dilate3x3(temp_binary, &temp_binary_1);
	Erode3x3(temp_binary_1, temp_binary_2);
	unsigned char *ptr = temp_binary_2->Begin();
	for (int ni=0 ; ni<img_width*img_height; ni++) 
	{
		if(*ptr++ == 255)
		{
			int rr = ni/(img_width);
			int cc = ni%(img_width);
			SeedPoint tmp;
			tmp.x = cc;
            tmp.y = rr;
			filled_clean_points->push_back(tmp);
		}
	}
}

void RegionPropsExtract(ImgGray& root_region, std::vector<SeedPoint>& filled_clean_points, RootElement* temp_region)
{
	ImgBinary root_binary;
	root_binary.Reset(root_region.Width(), root_region.Height());
	Convert(root_region, &root_binary);
	RegionProperties *props_1 = new RegionProperties[1];
	RegionProps(root_binary, props_1);
	double root_theta;
	if( (props_1->direction)*180/pi >= 0 )
	{
		root_theta = 180 - (props_1->direction)*180/pi;
	}
	if( (props_1->direction)*180/pi < 0 )
	{
		root_theta = -1.0*(props_1->direction)*180/pi;
	}
	temp_region->root_area = root_region;
	for(int kkk=0; kkk<filled_clean_points.size(); kkk++)
	{
	    temp_region->points_x.push_back(filled_clean_points[kkk].x);
	    temp_region->points_y.push_back(filled_clean_points[kkk].y);
	}
	temp_region->theta = root_theta;
	temp_region->eccentricity = props_1->eccentricity;
}

void BrokenRegionCombine(RootElement& root_old, ImgGray& img_1, ImgGray& img_2, std::vector<SeedPoint>& OldPoints, std::vector<SeedPoint>& NewPoints, ImgGray* img_3)
{
	ImgGray root_check, temp_root_check, total_temp_check, total_root_check;
	root_check.Reset(img_2.Width(), img_2.Height());
	temp_root_check.Reset(img_2.Width(), img_2.Height());
	total_root_check.Reset(img_2.Width(), img_2.Height());
	total_temp_check.Reset(img_2.Width(), img_2.Height());
	Set(&total_root_check, 255);
	Set(&total_temp_check, 255);
    
	int find_mid_point = 0;
	Or(img_1, img_2, &root_check);
	temp_root_check = root_check;

	ImgBinary root_binary_1;
	root_binary_1.Reset(img_1.Width(), img_1.Height());
	Convert(root_check, &root_binary_1);
	RegionProperties *props_1 = new RegionProperties[1];
	RegionProps(root_binary_1, props_1);
	int org_size = props_1->area;
	Rect boundary_before = props_1->bounding_rect;

	std::vector<double> line_dist, temp_line_dist;
	std::vector<SeedPoint> pos_loc;
	double temp_dist;
	for (int kk=0; kk< OldPoints.size(); kk++)
	{
		for (int kkk=0; kkk< NewPoints.size(); kkk++)
		{
			temp_dist = sqrt(pow((OldPoints[kk].x-NewPoints[kkk].x),2)+pow((OldPoints[kk].y-NewPoints[kkk].y),2));
            line_dist.push_back(temp_dist);
			SeedPoint tmp;
			tmp.x = kk;
			tmp.y =kkk;
			pos_loc.push_back(tmp);
		}
	}
	temp_line_dist = line_dist;
	sort(line_dist.begin(), line_dist.end());
	SeedPoint pos_1, pos_2;
	for(kk = 0; kk<temp_line_dist.size(); kk++)
	{
		if (temp_line_dist[kk] == line_dist[0])
		{
			pos_1.x = OldPoints[pos_loc[kk].x].x;
			pos_1.y = OldPoints[pos_loc[kk].x].y;
			pos_2.x = NewPoints[pos_loc[kk].y].x;
			pos_2.y = NewPoints[pos_loc[kk].y].y;
			break;
		}
	}
    SeedPoint temp_pos_1, temp_pos_2, temp_pos_3;
	temp_pos_1 = pos_1;
	temp_pos_2 = pos_2;
	if( root_old.total_line_para.size() == 1)
	{
	    temp_pos_3.x = floor((temp_pos_1.x+temp_pos_2.x)/2);
	    temp_pos_3.y = floor((temp_pos_1.y+temp_pos_2.y)/2);
	}
	else
	{
		double D1, D2, D;
		D = root_old.total_line_para[0].a*root_old.total_line_para[1].b - root_old.total_line_para[0].b*root_old.total_line_para[1].a;
		D1 = root_old.total_line_para[0].c*root_old.total_line_para[1].b - root_old.total_line_para[0].b*root_old.total_line_para[1].c;
		D2 = root_old.total_line_para[0].a*root_old.total_line_para[1].c - root_old.total_line_para[0].c*root_old.total_line_para[1].a;
		
		temp_pos_3.y = D1/D;
		temp_pos_3.x = D2/D;
		int temp_mid_3_x = floor((temp_pos_1.x+temp_pos_2.x)/2);
	    int temp_mid_3_y = floor((temp_pos_1.y+temp_pos_2.y)/2);
		double dist_1;
		dist_1 = sqrt(pow((temp_pos_3.x-temp_mid_3_x),2)+pow((temp_pos_3.y-temp_mid_3_y),2));

		if( (D1/D >= 0) && (D2/D >= 0) && (D1/D <= img_2.Height()) && (D2/D <= img_2.Width()) && (dist_1 <= 80) )
		{
		    temp_pos_3.y = D1/D;
		    temp_pos_3.x = D2/D;
		}
		else
		{
			temp_pos_3.x = floor((temp_pos_1.x+temp_pos_2.x)/2);
	        temp_pos_3.y = floor((temp_pos_1.y+temp_pos_2.y)/2);
		}
	}

	temp_dist = sqrt(pow((temp_pos_3.x-temp_pos_2.x),2)+pow((temp_pos_3.y-temp_pos_2.y),2));
	while ( (root_check(temp_pos_3.x, temp_pos_3.y)==255) && (temp_dist>=2) )
	{
		temp_pos_1 = temp_pos_3;
		temp_pos_3.x = floor((temp_pos_1.x+temp_pos_2.x)/2);
		temp_pos_3.y = floor((temp_pos_1.y+temp_pos_2.y)/2);
		temp_dist = sqrt(pow((temp_pos_3.x-temp_pos_2.x),2)+pow((temp_pos_3.y-temp_pos_2.y),2));
	}
	if(root_check(temp_pos_3.x, temp_pos_3.y)==0)
	{
		find_mid_point = 1;
		std::vector<int> NewRootPointsX;
	    std::vector<int> NewRootPointsY;
		std::vector<double> temp_line;
	    int new_width = ceil(root_old.width/2);

		for(int xx=0; xx<root_old.total_line_para.size(); xx++)
		{
			temp_line.clear();
			NewRootPointsX.clear();
			NewRootPointsY.clear();
			temp_line.push_back(root_old.total_line_para[xx].a);
			temp_line.push_back(root_old.total_line_para[xx].b);
			temp_line.push_back(root_old.total_line_para[xx].c);
			floodFill4Binary(root_check, temp_pos_3.x, temp_pos_3.y, 255, 0, new_width, temp_line, &NewRootPointsX, &NewRootPointsY);
			if(NewRootPointsX.size() > 1)
			{
				RootElement tmp;
				And(total_root_check, root_check, &total_root_check);
			}
			root_check = temp_root_check;
		}
	}
	else
	{
		temp_pos_1 = pos_1;
		temp_pos_2 = pos_2;
		if( root_old.total_line_para.size() == 1)
		{
	        temp_pos_3.x = floor((temp_pos_1.x+temp_pos_2.x)/2);
	        temp_pos_3.y = floor((temp_pos_1.y+temp_pos_2.y)/2);
		}
	    else
		{
		    double D1, D2, D;
		    D = root_old.total_line_para[0].a*root_old.total_line_para[1].b - root_old.total_line_para[0].b*root_old.total_line_para[1].a;
	    	D1 = root_old.total_line_para[0].c*root_old.total_line_para[1].b - root_old.total_line_para[0].b*root_old.total_line_para[1].c;
		    D2 = root_old.total_line_para[0].a*root_old.total_line_para[1].c - root_old.total_line_para[0].c*root_old.total_line_para[1].a;
		    
			temp_pos_3.y = D1/D;
		    temp_pos_3.x = D2/D;
			int temp_mid_3_x = floor((temp_pos_1.x+temp_pos_2.x)/2);
	        int temp_mid_3_y = floor((temp_pos_1.y+temp_pos_2.y)/2);
			double dist_1 = sqrt(pow((temp_pos_3.x-temp_mid_3_x),2)+pow((temp_pos_3.y-temp_mid_3_y),2));

			if( (D1/D >= 0) && (D2/D >= 0) && (D1/D <= img_2.Height()) && (D2/D <= img_2.Width()) && (dist_1 <= 80) )
			{
		        temp_pos_3.y = D1/D;
		        temp_pos_3.x = D2/D;
			}
		    else
			{
			    temp_pos_3.x = floor((temp_pos_1.x+temp_pos_2.x)/2);
	            temp_pos_3.y = floor((temp_pos_1.y+temp_pos_2.y)/2);
			}
		}
		temp_dist = sqrt(pow((temp_pos_3.x-temp_pos_1.x),2)+pow((temp_pos_3.y-temp_pos_1.y),2));
		while ( (root_check(temp_pos_3.x, temp_pos_3.y)==255) && (temp_dist>=2) )
		{
			temp_pos_2 = temp_pos_3;
			temp_pos_3.x = floor((temp_pos_1.x+temp_pos_2.x)/2);
			temp_pos_3.y = floor((temp_pos_1.y+temp_pos_2.y)/2);
			temp_dist = sqrt(pow((temp_pos_3.x-temp_pos_1.x),2)+pow((temp_pos_3.y-temp_pos_1.y),2));
		}
		if(root_check(temp_pos_3.x, temp_pos_3.y)==0)
		{
			find_mid_point = 1;
			std::vector<int> NewRootPointsX;
	        std::vector<int> NewRootPointsY;
		    std::vector<double> temp_line;
	        int new_width = ceil(root_old.width/2);
			for(int xx=0; xx<root_old.total_line_para.size(); xx++)
			{
				temp_line.clear();
				NewRootPointsX.clear();
				NewRootPointsY.clear();
				temp_line.push_back(root_old.total_line_para[xx].a);
				temp_line.push_back(root_old.total_line_para[xx].b);
				temp_line.push_back(root_old.total_line_para[xx].c);
				floodFill4Binary(root_check, temp_pos_3.x, temp_pos_3.y, 255, 0, new_width, temp_line, &NewRootPointsX, &NewRootPointsY);
				if(NewRootPointsX.size() > 1)
				{
				    RootElement tmp;
					And(total_root_check, root_check, &total_root_check);
				}
				root_check = temp_root_check;
			}
		}
	}
    
	if( find_mid_point == 1 )
	{
	    Dilate3x3(total_root_check, &total_root_check);
	    Erode3x3(total_root_check, &total_root_check);

	    Dilate3x3(total_root_check, &total_root_check);
	    Erode3x3(total_root_check, &total_root_check);
        
		ImgBinary root_binary_2;
	    root_binary_2.Reset(img_1.Width(), img_1.Height());
	    Convert(total_root_check, &root_binary_2);
	    RegionProperties *props_2 = new RegionProperties[1];
	    RegionProps(root_binary_2, props_2);
	    int new_size = props_2->area;
		Rect boundary_after = props_2->bounding_rect;
        double scale_size = double(new_size)/(org_size);
		int left_right = 0;
		int top_bottom = 0;
		if( (abs(boundary_after.left-boundary_before.left) <=5) && (abs(boundary_after.right-boundary_before.right)<=5) )
		{
			left_right = 1;
		}
		if( (abs(boundary_after.bottom-boundary_before.bottom) <=5) && (abs(boundary_after.top-boundary_before.top)<=5) )
		{
			top_bottom = 1;
		}
		int good_combine = 0;
		if( (top_bottom == 1) || (left_right == 1) )
		{
			good_combine = 1;
		}

		if ( (good_combine == 1) && (IsIdentical(total_temp_check, total_root_check) == 0) && (scale_size <= 2.4) )
		{
	        *(img_3) = total_root_check;
		}
	    else
		{
		    *(img_3) = temp_root_check;
		}
	}
	else
	{
		*(img_3) = temp_root_check;
	}
}


void RootsRegionGrowing(const std::vector<RootElement>& clean_root, ImgGray& GreyImg, std::vector<RootElement>* grown_root)
{
	ImgGray GreyImgHalf, GreyImgSmoothed;
	SmoothGauss5x5(GreyImg, &GreyImgSmoothed);
	Downsample(GreyImgSmoothed, 2, 2, &GreyImgHalf);
	int ImgWidth = GreyImgHalf.Width();
	int ImgHeight = GreyImgHalf.Height();
    
	std::vector<int> FilledPointX;
    std::vector<int> FilledPointY;
	std::vector<SeedPoint> CoverPoints;
	std::vector<SeedPoint> FilledCleanPoints;
	std::vector<SeedPoint> OldFilledCleanPoints;
	std::vector<SeedPoint> NotCoverPoints;
	std::vector<SeedPoint> OldNotCoverPoints;
	std::vector<SeedPoint> OldCoverPoints;
	std::vector<int> grey_val;
	for(int ii=0; ii<clean_root.size(); ii++)
	{
		RootElement RootOne = clean_root[ii];
		ImgGray img_binary_1, img_binary_2, img_binary_3;
		double include_ratio = 0.0;
		FilledPointX.clear();
		FilledPointY.clear();
		CoverPoints.clear();
		NotCoverPoints.clear();
		grey_val.clear();
		for(int nnn=0; nnn<clean_root[ii].points_x.size(); nnn++)
		{
			grey_val.push_back(GreyImgHalf(clean_root[ii].points_x[nnn], clean_root[ii].points_y[nnn]));	
		}
		sort(grey_val.begin(), grey_val.end());
		if( double(grey_val[0]) < RootOne.val )
		{
			RootOne.val = double(grey_val[0]);
		}
        GreyFloodFill(RootOne, GreyImgHalf, &include_ratio, &CoverPoints, &NotCoverPoints, &FilledPointX, &FilledPointY);
	    OldCoverPoints = CoverPoints;
		OldNotCoverPoints = NotCoverPoints;
		
		if( include_ratio >= 0.45 )
		{
			FilledCleanPoints.clear();
			img_binary_1.Reset(ImgWidth, ImgHeight);
			RegionClean(FilledPointX, FilledPointY, ImgWidth, ImgHeight, &img_binary_1, &FilledCleanPoints);

			if (FilledCleanPoints.size() >= 125)
			{   
				double diameter_1;
				int salient_1 = 0;
				FindDiameter(img_binary_1, CoverPoints, &diameter_1, &salient_1);
				RootOne.width = diameter_1;
                
                    FilledPointX.clear();
		            FilledPointY.clear();
		            CoverPoints.clear();
		            NotCoverPoints.clear();
				    FilledCleanPoints.clear();
				    img_binary_1.Reset(ImgWidth, ImgHeight);
				    GreyFloodFill(RootOne, GreyImgHalf, &include_ratio, &CoverPoints, &NotCoverPoints, &FilledPointX, &FilledPointY);
				    RegionClean(FilledPointX, FilledPointY, ImgWidth, ImgHeight, &img_binary_1, &FilledCleanPoints);
				    OldFilledCleanPoints.clear();
				    OldFilledCleanPoints = FilledCleanPoints;
			}
          
			double sec_include_ratio = 0.0;
			if( OldNotCoverPoints.size() <= 4)
			{
				RootElement tmp_region_1;
				tmp_region_1.valid_points_x = clean_root[ii].valid_points_x;
				tmp_region_1.valid_points_y = clean_root[ii].valid_points_y;
				tmp_region_1.end_points = clean_root[ii].end_points;
				RegionPropsExtract(img_binary_1, FilledCleanPoints, &tmp_region_1);
				tmp_region_1.width = RootOne.width;
				tmp_region_1.id = ii;
				if (FilledCleanPoints.size() >= 50)
				{
				    grown_root->push_back(tmp_region_1);
				}
				
			}
		    else 
			{
				RootOne.points_x.clear();
			    RootOne.points_y.clear();
			    for(int iii=0; iii<OldNotCoverPoints.size(); iii++)
				{
				    RootOne.points_x.push_back(OldNotCoverPoints[iii].x);
				    RootOne.points_y.push_back(OldNotCoverPoints[iii].y);
				}
			    FilledPointX.clear();
		        FilledPointY.clear();
		        CoverPoints.clear();
		        NotCoverPoints.clear();
			    GreyFloodFill(RootOne, GreyImgHalf, &sec_include_ratio, &CoverPoints, &NotCoverPoints, &FilledPointX, &FilledPointY);

				if(sec_include_ratio >= 0.5)
				{
				    FilledCleanPoints.clear();
			        img_binary_2.Reset(ImgWidth, ImgHeight);
			        RegionClean(FilledPointX, FilledPointY, ImgWidth, ImgHeight, &img_binary_2, &FilledCleanPoints);
					if (FilledCleanPoints.size() >= 125)
					{   
						double diameter_2;
						int salient_2 = 0;
						FindDiameter(img_binary_2, CoverPoints, &diameter_2, &salient_2);
						if(RootOne.width > diameter_2)
						{
							RootOne.width = diameter_2;
						}
						ImgGray img_binary_3, img_binary_4;
						BrokenRegionCombine(RootOne, img_binary_1, img_binary_2, OldCoverPoints, CoverPoints, &img_binary_3);
                        
						ImgInt img_lables;
						ConnectedComponents(img_binary_3, &img_lables);
						int componet_no = Max(img_lables);
						for(int xxx =1; xxx<=componet_no; xxx++)
						{
							ImgGray new_root;
							new_root.Reset(ImgWidth, ImgHeight);
							Set(&new_root, 0);
							FilledCleanPoints.clear();
							unsigned char *ptr1 = new_root.Begin();
							int *ptr = img_lables.Begin();
							for (int ni=0 ; ni<ImgWidth*ImgHeight; ni++) 
							{
								if(*ptr++ == xxx)
								{
									*ptr1 = 255;
									int rr = ni/(ImgWidth);
							        int cc = ni%(ImgHeight);
							    	SeedPoint tmp;
							    	tmp.x = cc;
							    	tmp.y = rr;
							    	FilledCleanPoints.push_back(tmp);
								}
									*ptr1++;
							}
							if (FilledCleanPoints.size() >= 125)
							{
								RootElement tmp_region_1;
								tmp_region_1.end_points = clean_root[ii].end_points;
								tmp_region_1.valid_points_x = clean_root[ii].valid_points_x;
				                tmp_region_1.valid_points_y = clean_root[ii].valid_points_y;
							    RegionPropsExtract(new_root, FilledCleanPoints, &tmp_region_1);
							    tmp_region_1.width = RootOne.width;
							    tmp_region_1.id = ii;
								grown_root->push_back(tmp_region_1);
							}
						}
					}
					else
					{
					    RootElement tmp_region_1;
						tmp_region_1.end_points = clean_root[ii].end_points;
						tmp_region_1.valid_points_x = clean_root[ii].valid_points_x;
				        tmp_region_1.valid_points_y = clean_root[ii].valid_points_y;
				        RegionPropsExtract(img_binary_1, OldFilledCleanPoints, &tmp_region_1);
						tmp_region_1.id = ii;
						tmp_region_1.width = RootOne.width;
						if (OldFilledCleanPoints.size() >= 125)
						{
				            grown_root->push_back(tmp_region_1);
						}
					}
				}
				else
				{
					RootElement tmp_region_1;
					tmp_region_1.end_points = clean_root[ii].end_points;
					tmp_region_1.valid_points_x = clean_root[ii].valid_points_x;
				    tmp_region_1.valid_points_y = clean_root[ii].valid_points_y;
				    RegionPropsExtract(img_binary_1, OldFilledCleanPoints, &tmp_region_1);
					tmp_region_1.width = RootOne.width;
					tmp_region_1.id = ii;
				    if (OldFilledCleanPoints.size() >= 125)
					{
						grown_root->push_back(tmp_region_1);
					}
				}
			}
		}	
	}
}


void LineFitting(const MatDbl& line_points, std::vector<double>* line_para)
{
	    MatDbl line_points_transpose;
		Transpose(line_points, &line_points_transpose); 
		MatDbl ones;
		ones.Reset(1,line_points.Height());
		for( int ii = 0; ii<line_points.Height(); ii++ )
		{
			ones(0,ii) = 1;
		}
		MatDbl p, p_transpose, q;
		MatDbl u, s, v;

		p = line_points_transpose*ones;
		MatDbl ppp;
		double m;
		m = 1/double(line_points.Height());
		Multiply(p, m, &ppp);
		
        Transpose(ppp, &p_transpose);
        Subtract(line_points, ones*p_transpose, &q);
		Svd(q, &u, &s, &v);
		MatDbl dd;
		dd.Reset(1,2);
		dd(0,0) = v(1,0);
		dd(0,1) = v(1,1);
		MatDbl cc;
        cc = p_transpose*dd;
		line_para->push_back(dd(0,0));
		line_para->push_back(dd(0,1));
		line_para->push_back(cc(0,0));
		line_para->push_back(s(1));
}

void RootSelect(const std::vector<SeedPoint>& seed_pp, std::vector<RootElement>* points_pair)
{
	std::vector<int> points_uu;
	for (int ii = 0; ii<seed_pp.size(); ii++)
	{
		points_uu.push_back(0);
	}

	std::vector<double> points_dist;
	std::vector<int> points_dist_x;
	std::vector<int> points_dist_y;

	std::vector<double> temp_points_dist;
	std::vector<int> temp_points_dist_x;
	std::vector<int> temp_points_dist_y;

	std::vector<int> temp_dist_x;
	std::vector<int> temp_dist_y;
	
	std::vector<int> temp_pp;
	std::vector<double> line_coef;

	MatDbl new_line_points;
	new_line_points.Reset(2,2);

	for (ii=0; ii<seed_pp.size(); ii++)
	{   
		points_dist.clear();
		points_dist_x.clear();
		points_dist_y.clear();
		temp_points_dist.clear();
		temp_points_dist_x.clear();
		temp_points_dist_y.clear();
		temp_dist_x.clear();
		temp_dist_y.clear();

		for(int jj=0; jj<seed_pp.size(); jj++)
		{
			if ( (ii != jj) && (points_uu[ii] == 0) && (points_uu[jj] == 0) )
			{
				double dist_val = 0.0;
				dist_val = sqrt( pow((seed_pp[ii].x-seed_pp[jj].x), 2) + pow((seed_pp[ii].y-seed_pp[jj].y), 2) );
				points_dist.push_back(dist_val);
				points_dist_x.push_back(seed_pp[jj].x);
				points_dist_y.push_back(seed_pp[jj].y);
			}
		}

		temp_points_dist = points_dist;
	    sort(temp_points_dist.begin(), temp_points_dist.end());
	    for(int iii=0; iii<temp_points_dist.size(); iii++)
		{
		    for(int jjj=0; jjj<points_dist.size(); jjj++)
			{
			    if( (temp_points_dist[iii] == points_dist[jjj]) && (points_dist_x[jjj] != 9999))
				{
				    temp_points_dist_x.push_back(points_dist_x[jjj]);
				    temp_points_dist_y.push_back(points_dist_y[jjj]);
				    points_dist_x[jjj] =9999;
				}
			}
		}

		temp_pp.clear();
		for (iii = 0; iii<temp_points_dist_x.size(); iii++)
		{
		    temp_pp.push_back(0);
		}

		for(jj=0; jj<temp_points_dist_x.size(); jj++)
		{
			double temp_dd = sqrt( pow((seed_pp[ii].x-temp_points_dist_x[jj]), 2) + pow((seed_pp[ii].y-temp_points_dist_y[jj]), 2) );
			double temp_dist_val = 0.0;
            int temp_dist_number = 0;
			if( (temp_dd <= 25.0) && (temp_pp[jj] == 0) )
			{
    		    line_coef.clear();
                temp_dist_x.clear();
		        temp_dist_y.clear();

				temp_dist_x.push_back(seed_pp[ii].x);
				temp_dist_y.push_back(seed_pp[ii].y);
				temp_dist_x.push_back(temp_points_dist_x[jj]);
				temp_dist_y.push_back(temp_points_dist_y[jj]);

	        	new_line_points.Reset(2,2);
				new_line_points(0,0) = seed_pp[ii].y;
		        new_line_points(1,0) = seed_pp[ii].x;
		        new_line_points(0,1) = temp_points_dist_y[jj];
		        new_line_points(1,1) = temp_points_dist_x[jj];
                line_coef.clear();
                LineFitting(new_line_points, &line_coef);

			    int new_x = temp_points_dist_x[jj];
			    int new_y = temp_points_dist_y[jj];
			    for ( int kk = 0; kk < temp_points_dist_x.size(); kk++ )
				{
					if( (kk != ii) && (kk != jj) && (temp_pp[kk] == 0) )
					{
					    int next_x = temp_points_dist_x[kk];
                        int next_y = temp_points_dist_y[kk];
					    temp_dd = sqrt( pow((new_x-next_x), 2) + pow((new_y-next_y), 2) );
					    double temp_down = pow(line_coef[0], 2) + pow(line_coef[1], 2);
				    	double temp_up = abs(line_coef[0]*temp_points_dist_y[kk] + line_coef[1]*temp_points_dist_x[kk] - line_coef[2]);
					    temp_dist_val = temp_up/sqrt(temp_down);
					    if ( (temp_dist_val <= 3.0) && (temp_dd <= 25.0) )
						{
							temp_dist_number = temp_dist_number + 1;
						    temp_dist_x.push_back(temp_points_dist_x[kk]);
						    temp_dist_y.push_back(temp_points_dist_y[kk]);
						    new_x = temp_points_dist_x[kk];
					        new_y = temp_points_dist_y[kk];
						}
					}
				}
			
			    if ( temp_dist_x.size() >= 5 )
				{   
				    line_coef.clear();
				    MatDbl refined_line_points;
		            refined_line_points.Reset(2,temp_dist_x.size());
				    for (int kkk=0; kkk<temp_dist_x.size(); kkk++)
					{
					    refined_line_points(0,kkk) = temp_dist_y[kkk];
					    refined_line_points(1,kkk) = temp_dist_x[kkk];
					}
				    LineFitting(refined_line_points, &line_coef);
                    
					RootElement tmp;
					tmp.line_parameter = line_coef;
			        tmp.length = temp_dist_x.size();
			        tmp.theta = 0;
			        //tmp.points_x = temp_dist_x;
			        //tmp.points_y = temp_dist_y;
					tmp.points_x = temp_dist_x;
			        tmp.points_y = temp_dist_y;
					double slope;
                    slope = (-1.0*line_coef[0])/line_coef[1];
					tmp.theta = abs(90.0 + atan(slope)*180.0/pi);
			        points_pair->push_back(tmp);

				    for ( int kt=0; kt<temp_dist_x.size(); kt++)
					{
					    for (int kkt=0; kkt<seed_pp.size(); kkt++)
						{
						    if( (temp_dist_x[kt]==seed_pp[kkt].x) && (temp_dist_y[kt]==seed_pp[kkt].y) )
							{
							    points_uu[kkt] = 1;
							}
						}
						for (kkt=0; kkt<temp_points_dist_x.size(); kkt++)
						{
						    if( (temp_dist_x[kt]==temp_points_dist_x[kkt]) && (temp_dist_y[kt]==temp_points_dist_y[kkt]) )
							{
							    temp_pp[kkt] = 1;
							}
						}
					}
					temp_dist_x.clear();
		            temp_dist_y.clear();
				}
				else
				{
					temp_dist_x.clear();
		            temp_dist_y.clear();
				}
			
			}
		}
    }
	
}

void FindTwoEnd(RootElement& line_1, RootElement& line_2, std::vector<SeedPoint>& one_end, std::vector<SeedPoint>* two_end)
{
	SeedPoint tmp;
	tmp.x = one_end[0].x;
	tmp.y = one_end[0].y;
    two_end->push_back(tmp);
	double temp_dist;
	double temp_max = 0.0;
	int temp_max_id = 0;
	for (int kkk=0; kkk< line_1.points_x.size(); kkk++)
	{
		temp_dist = sqrt(pow((line_1.points_x[kkk]-tmp.x),2)+pow((line_1.points_y[kkk]-tmp.y),2)); 
		if( temp_dist > temp_max )
		{
			temp_max = temp_dist;
			temp_max_id = kkk;
		}
	}
	tmp.x = line_1.points_x[temp_max_id];
	tmp.y = line_1.points_y[temp_max_id];
    two_end->push_back(tmp);
	//line_1.end_points.push_back(tmp);

	/*temp_max = 0.0;
	temp_max_id = 0;
	for (kkk=0; kkk< line_1.points_x.size(); kkk++)
	{
		temp_dist = sqrt(pow((line_1.points_x[kkk]-tmp.x),2)+pow((line_1.points_y[kkk]-tmp.y),2)); 
		if( temp_dist > temp_max )
		{
			temp_max = temp_dist;
			temp_max_id = kkk;
		}
	}
	tmp.x = line_1.points_x[temp_max_id];
	tmp.y = line_1.points_y[temp_max_id];
	line_1.end_points.push_back(tmp);*/

	tmp.x = one_end[1].x;
	tmp.y = one_end[1].y;
	two_end->push_back(tmp);
	temp_max = 0.0;
    temp_max_id = 0;
	for (kkk=0; kkk< line_2.points_x.size(); kkk++)
	{
		temp_dist = sqrt(pow((line_2.points_x[kkk]-tmp.x),2)+pow((line_2.points_y[kkk]-tmp.y),2)); 
		if( temp_dist > temp_max )
		{
			temp_max = temp_dist;
			temp_max_id = kkk;
		}
	}
	tmp.x = line_2.points_x[temp_max_id];
	tmp.y = line_2.points_y[temp_max_id];
    two_end->push_back(tmp);
	//line_2.end_points.push_back(tmp);

	/*temp_max = 0.0;
	temp_max_id = 0;
	for (kkk=0; kkk< line_2.points_x.size(); kkk++)
	{
		temp_dist = sqrt(pow((line_2.points_x[kkk]-tmp.x),2)+pow((line_2.points_y[kkk]-tmp.y),2)); 
		if( temp_dist > temp_max )
		{
			temp_max = temp_dist;
			temp_max_id = kkk;
		}
	}
	tmp.x = line_2.points_x[temp_max_id];
	tmp.y = line_2.points_y[temp_max_id];
	line_2.end_points.push_back(tmp);*/
}

void LineDist(RootElement& line_1, RootElement& line_2, double* dist_lines)
{
	std::vector<double> line_dist;
	double temp_dist;
	for (int kk=0; kk< line_1.points_x.size(); kk++)
	{
		for (int kkk=0; kkk< line_2.points_x.size(); kkk++)
		{
			temp_dist = sqrt(pow((line_1.points_x[kk]-line_2.points_x[kkk]),2)+pow((line_1.points_y[kk]-line_2.points_y[kkk]),2));
            line_dist.push_back(temp_dist);
		}
	}
	sort(line_dist.begin(), line_dist.end());
	(*dist_lines) = line_dist[0];
}

void RootsCombine(std::vector<RootElement>& clean_roots, std::vector<RootElement>* combined_roots)
{

	for (int iii=0; iii<clean_roots.size(); iii++)
	{
		clean_roots[iii].id = iii;
	}

	std::vector<double> dd_val;
	std::vector<double> d_val;
	std::vector<double> temp_dd_val;
	std::vector<double> temp_d_val;
	EquivalenceTable equiv_table;
	std::vector<SeedPoint> end_points, two_end_points;
 	for(int kk = 0; kk<clean_roots.size()-1; kk++)
	{
		double intersect_points_x, intersect_points_y;
		for(int kkk = kk+1; kkk<clean_roots.size(); kkk++)
		{
			end_points.clear();
			two_end_points.clear();
			double D1, D2, D;
			D = clean_roots[kk].line_parameter[0]*clean_roots[kkk].line_parameter[1]-clean_roots[kk].line_parameter[1]*clean_roots[kkk].line_parameter[0];
			D1 = clean_roots[kk].line_parameter[2]*clean_roots[kkk].line_parameter[1]-clean_roots[kk].line_parameter[1]*clean_roots[kkk].line_parameter[2];
			D2 = clean_roots[kk].line_parameter[0]*clean_roots[kkk].line_parameter[2]-clean_roots[kk].line_parameter[2]*clean_roots[kkk].line_parameter[0];
		
			intersect_points_y = D1/D;
			intersect_points_x = D2/D;
			if( (intersect_points_x >= 0.0) && (intersect_points_y >= 0.0) && (intersect_points_x <= 320.0) && (intersect_points_y <= 240.0) )
			{
				SeedPoint tmp;
				dd_val.clear();
				d_val.clear();
				temp_dd_val.clear();
				temp_d_val.clear();
				for(int xx=0; xx<clean_roots[kk].points_x.size(); xx++)
				{
					double pp_dist = sqrt(pow((intersect_points_x-clean_roots[kk].points_x[xx]),2)+pow((intersect_points_y-clean_roots[kk].points_y[xx]),2));
				    dd_val.push_back(pp_dist);
				}
				temp_dd_val = dd_val;
				sort(temp_dd_val.begin(), temp_dd_val.end());
				int end_points_1_x, end_points_1_y;
				for(int jjj=0; jjj<dd_val.size(); jjj++)
				{
					if( temp_dd_val[dd_val.size()-1] == dd_val[jjj] )
					{
						end_points_1_x = clean_roots[kk].points_x[jjj];
						end_points_1_y = clean_roots[kk].points_y[jjj];
					}
				}
				tmp.x = end_points_1_x;
				tmp.y = end_points_1_y;
				end_points.push_back(tmp);
				for(xx=0; xx<clean_roots[kkk].points_x.size(); xx++)
				{
					double p_dist = sqrt(pow((intersect_points_x-clean_roots[kkk].points_x[xx]),2)+pow((intersect_points_y-clean_roots[kkk].points_y[xx]),2));
				    d_val.push_back(p_dist);
				}
				temp_d_val = d_val;
				sort(temp_d_val.begin(), temp_d_val.end());
				int end_points_2_x, end_points_2_y;
				for(jjj=0; jjj<d_val.size(); jjj++)
				{
					if( temp_d_val[d_val.size()-1] == d_val[jjj] )
					{
						end_points_2_x = clean_roots[kkk].points_x[jjj];
						end_points_2_y = clean_roots[kkk].points_y[jjj];
					}
				}
				tmp.x = end_points_2_x;
				tmp.y = end_points_2_y;
				end_points.push_back(tmp);
				  
				FindTwoEnd(clean_roots[kk], clean_roots[kkk], end_points, &two_end_points);
				double dist_a, dist_aa, dist_b, dist_bb, dist_c;
				dist_a = pow((intersect_points_x-two_end_points[0].x),2)+pow((intersect_points_y-two_end_points[0].y),2);
				dist_aa = pow((intersect_points_x-two_end_points[1].x),2)+pow((intersect_points_y-two_end_points[1].y),2);
				dist_b = pow((intersect_points_x-two_end_points[2].x),2)+pow((intersect_points_y-two_end_points[2].y),2);
				dist_bb = pow((intersect_points_x-two_end_points[3].x),2)+pow((intersect_points_y-two_end_points[3].y),2);
				dist_c = pow((two_end_points[0].x-two_end_points[2].x),2)+pow((two_end_points[0].y-two_end_points[2].y),2);
				double angle_val;
				angle_val = acos((dist_a + dist_b - dist_c)/(2*sqrt(dist_a*dist_b)))*180/pi;
				double dist_ratio_1, dist_ratio_2;
				if(dist_a > dist_aa )
				{
					dist_ratio_1 = dist_a/dist_aa;
				}
				else
				{
					dist_ratio_1 = dist_aa/dist_a;
				}
				if(dist_b > dist_bb )
				{
					dist_ratio_2 = dist_b/dist_bb;
				}
				else
				{
					dist_ratio_2 = dist_bb/dist_b;
				}
				int Junction_T = 0;
				if( (dist_ratio_2 <= 2.0) || (dist_ratio_1 <= 2.0) )
				{
					Junction_T = 1;
				}
			    
				double dist_mean_points;
			    LineDist(clean_roots[kkk], clean_roots[kk], &dist_mean_points);
                
				double dist_exp = exp(-1.0*(60.0-dist_mean_points)/60.0);
				double angle_exp =(180-angle_val)/180.0;
				double total_exp = angle_exp*dist_exp;
				//if ( (total_exp <= 0.122) && (Junction_T ==0) )	
				if ( ((angle_val <= 2.0) || (angle_val >= 135.0)) && (dist_mean_points <= 65.0) && (Junction_T ==0) )
				{
				    equiv_table.AddEquivalence(kk, kkk);
				} 
			    else
				{
				    equiv_table.AddEquivalence(kk, kk);
				    equiv_table.AddEquivalence(kkk, kkk);
				}
			}
			else
			{
				equiv_table.AddEquivalence(kk, kk);
				equiv_table.AddEquivalence(kkk, kkk);
			}
		}
	}

	equiv_table.TraverseLinks();
	int label_no = equiv_table.RemoveGaps();
    
	for (int ii=0; ii<clean_roots.size(); ii++)
	{ 
		clean_roots[ii].id = equiv_table.GetEquivalentLabel(ii);
	}

	for(ii=0; ii<label_no; ii++)
	{
		RootElement tmp;
		double temp_width_min = 9999.0;
		double temp_val_min = 9999.0;
		for(int jj=0; jj<clean_roots.size(); jj++)
		{
			if( clean_roots[jj].id == ii )
			{
				for(int nnn = 0; nnn<clean_roots[jj].points_x.size(); nnn++)
				{
					tmp.points_x.push_back(clean_roots[jj].points_x[nnn]);
					tmp.points_y.push_back(clean_roots[jj].points_y[nnn]);
					tmp.valid_points_x.push_back(clean_roots[jj].valid_points_x[nnn]);
					tmp.valid_points_y.push_back(clean_roots[jj].valid_points_y[nnn]);
				}
				if(clean_roots[jj].width < temp_width_min)
				{
					temp_width_min = clean_roots[jj].width;
				}
				if(clean_roots[jj].val < temp_val_min)
				{
					temp_val_min = clean_roots[jj].val;
				}
				LineParameter tmp_line;
				tmp_line.a = clean_roots[jj].line_parameter[0];
				tmp_line.b = clean_roots[jj].line_parameter[1];
				tmp_line.c = clean_roots[jj].line_parameter[2];
				tmp_line.d = clean_roots[jj].line_parameter[3];
				tmp.total_line_para.push_back(tmp_line);
				tmp.points_no.push_back(clean_roots[jj].points_x.size());
				SeedPoint tmp_end;
				for (int xxx = 0; xxx <= 1; xxx++)
				{
					tmp_end.x = clean_roots[jj].end_points[xxx].x;
					tmp_end.y = clean_roots[jj].end_points[xxx].y;
					tmp.end_points.push_back(tmp_end);
				}
				
			}
		}
		tmp.width = temp_width_min;
		tmp.val = temp_val_min;
		combined_roots->push_back(tmp);
	}
}


void BorderCheck(RootElement& tmp, int* valid)
{
	int left_border, right_border, top_border, bottom_border;
	left_border = 50;
	right_border = 270;
	top_border = 40,
	bottom_border = 200;
	*(valid) = 1;
    
	int points_sum_x = 0;
	int points_sum_y = 0;
    
	int points_mean_x = 0;
	int points_mean_y = 0;

	for (int kkk=0; kkk<tmp.points_x.size(); kkk++)
	{
		points_sum_x = points_sum_x + tmp.points_x[kkk];
		points_sum_y = points_sum_y + tmp.points_y[kkk];
	}
	points_mean_x = points_sum_x/tmp.points_x.size();
	points_mean_y = points_sum_y/tmp.points_y.size();
    
	if( ((points_mean_x < left_border) || (points_mean_x > right_border)) && (abs(tmp.theta-90) <= 15) )
	{
		*(valid) =0; 
	}

	if( ((points_mean_y < top_border) || (points_mean_y > bottom_border)) && ((abs(tmp.theta-180) <= 15) || (tmp.theta <= 15)) )
	{
		*(valid) =0; 
	}

}

void FindTwoEnd(RootElement& line_1, std::vector<SeedPoint>* two_end)
{
	int temp_end_x = line_1.valid_points_x[0];
	int temp_end_y = line_1.valid_points_y[0];
	
	SeedPoint tmp;
	double temp_dist;
	double temp_max = 0.0;
	int temp_max_id = 0;
	for (int kkk=1; kkk< line_1.valid_points_x.size(); kkk++)
	{
		temp_dist = sqrt(pow((line_1.valid_points_x[kkk]-temp_end_x),2)+pow((line_1.valid_points_y[kkk]-temp_end_y),2)); 
		if( temp_dist > temp_max )
		{
			temp_max = temp_dist;
			temp_max_id = kkk;
		}
	}
	tmp.x = line_1.valid_points_x[temp_max_id];
	tmp.y = line_1.valid_points_y[temp_max_id];
    two_end->push_back(tmp);

	temp_end_x = tmp.x;
	temp_end_y = tmp.y;
	temp_max = 0.0;
	temp_max_id = 0;
	for (kkk=0; kkk< line_1.valid_points_x.size(); kkk++)
	{
		temp_dist = sqrt(pow((line_1.valid_points_x[kkk]-temp_end_x),2)+pow((line_1.valid_points_y[kkk]-temp_end_y),2)); 
		if( temp_dist > temp_max )
		{
			temp_max = temp_dist;
			temp_max_id = kkk;
		}
	}
	tmp.x = line_1.valid_points_x[temp_max_id];
	tmp.y = line_1.valid_points_y[temp_max_id];
    two_end->push_back(tmp);
}


void RootsClean(std::vector<RootElement>& points_pair, std::vector<SeedPoint>& clean_seed, std::vector<RootElement>* root_points)
{
	std::vector<double> line_coef;
	std::vector<double> line_index;
	std::vector<double> root_seed_val;
	std::vector<int> root_seed_width;
	EquivalenceTable equiv_table;
    double dist_avg = 0.0;
    
    if(points_pair.size() == 1)
    {
	    RootElement tmp;
		for(int kk = 0; kk<points_pair[0].points_x.size(); kk++)
		{
		    for(int qq=0; qq<clean_seed.size(); qq++)
			{
			    if((points_pair[0].points_x[kk] == clean_seed[qq].x) && (points_pair[0].points_y[kk] == clean_seed[qq].y))
				{
				    root_seed_val.push_back(clean_seed[qq].val);
				    root_seed_width.push_back(clean_seed[qq].width);
				}
			}
		    sort(root_seed_val.begin(), root_seed_val.end());
		    sort(root_seed_width.begin(), root_seed_width.end());
		    int median_loc = floor(root_seed_width.size()/2);
		    if( 2*median_loc != root_seed_width.size() )
			{
			    tmp.width = root_seed_width[median_loc]/2;
			}
		    else
			{
			    tmp.width = (root_seed_width[median_loc] + root_seed_width[median_loc-1])/4;
			}
		    tmp.val = root_seed_val[0];
		    tmp.points_x.push_back(floor(points_pair[0].points_x[kk]/2));
		    tmp.points_y.push_back(floor(points_pair[0].points_y[kk]/2));
			tmp.valid_points_x.push_back(points_pair[0].points_x[kk]);
		    tmp.valid_points_y.push_back(points_pair[0].points_y[kk]);

		}
		line_index.clear();
		MatDbl tt_line_points;
		tt_line_points.Reset(2,tmp.points_x.size());
		for (int kkk=0; kkk<tmp.points_x.size(); kkk++)
		{
			tt_line_points(0,kkk) = tmp.points_y[kkk];
			tt_line_points(1,kkk) = tmp.points_x[kkk];
		}
		LineFitting(tt_line_points, &line_index);
		double slope;
		slope = (-1.0*line_index[0])/line_index[1];
		tmp.theta = 90.0 + atan(slope)*180.0/pi;
		tmp.line_parameter = line_index;
		tmp.id = 0;
		int good_root;
		BorderCheck(tmp, &good_root);
		if( good_root == 1)
		{
		    std::vector<SeedPoint> line_ends;
			FindTwoEnd(tmp, &line_ends);
            tmp.end_points = line_ends;
			root_points->push_back(tmp);
		}
	}
    else
	{
		for (int iii=0; iii<points_pair.size(); iii++)
		{
		    points_pair[iii].id = iii;
		}

	    int line_points_1, line_points_2;
        for (int ii=0; ii<points_pair.size()-1; ii++)
		{
		    dist_avg = 0.0;
		    line_points_1 = points_pair[ii].points_x.size();
		    for (int jj=ii+1; jj<points_pair.size(); jj++)
			{
                double dist_mean_points;
			    LineDist(points_pair[ii], points_pair[jj], &dist_mean_points);
			    line_points_2 = points_pair[jj].points_x.size();
		        MatDbl refined_line_points;
		        refined_line_points.Reset(2,line_points_2+line_points_1);
		        for (int kkk=0; kkk< line_points_1; kkk++)
				{
				    refined_line_points(0,kkk) = points_pair[ii].points_y[kkk];
				    refined_line_points(1,kkk) = points_pair[ii].points_x[kkk];
				}
            
			    for (kkk=0; kkk< line_points_2; kkk++)
				{
				    refined_line_points(0,kkk+line_points_1) = points_pair[jj].points_y[kkk];
				    refined_line_points(1,kkk+line_points_1) = points_pair[jj].points_x[kkk];
				}
			    line_coef.clear();
			    LineFitting(refined_line_points, &line_coef);
			    dist_avg = line_coef[3]/(line_points_2+line_points_1);
				double dist_exp = exp(-1.0*(120.0-dist_mean_points)/120.0)*dist_avg;
				//if (dist_exp <= 0.34)
		        if ( (dist_avg <= 0.57) && (dist_mean_points <= 130.0) )
				{
				    equiv_table.AddEquivalence(ii, jj);
				} 
			    else
				{
				    equiv_table.AddEquivalence(ii, ii);
				    equiv_table.AddEquivalence(jj, jj);
				}
			}
		}
	    equiv_table.TraverseLinks();
    	int label_no = equiv_table.RemoveGaps();
    
    	for (ii=0; ii<points_pair.size(); ii++)
		{ 
		    points_pair[ii].id = equiv_table.GetEquivalentLabel(ii);
		}

    	for(ii=0; ii<label_no; ii++)
		{
		    RootElement tmp;
            root_seed_val.clear();
	    	root_seed_width.clear();
	    	for(int jj=0; jj<points_pair.size(); jj++)
			{
			    if( points_pair[jj].id == ii )
				{
				    for(int kk = 0; kk<points_pair[jj].points_x.size(); kk++)
					{
				        for(int qq=0; qq<clean_seed.size(); qq++)
						{
						    if((points_pair[jj].points_x[kk] == clean_seed[qq].x) && (points_pair[jj].points_y[kk] == clean_seed[qq].y))
							{
							    root_seed_val.push_back(clean_seed[qq].val);
							    root_seed_width.push_back(clean_seed[qq].width);
							}
						}
					    sort(root_seed_val.begin(), root_seed_val.end());
			            sort(root_seed_width.begin(), root_seed_width.end());
					    int median_loc = floor(root_seed_width.size()/2);
			            if( 2*median_loc != root_seed_width.size() )
						{
				            tmp.width = root_seed_width[median_loc]/2;
						}
			            else
						{
				            tmp.width = (root_seed_width[median_loc] + root_seed_width[median_loc-1])/4;
						}
                        tmp.val = root_seed_val[0];
					    tmp.points_x.push_back(floor(points_pair[jj].points_x[kk]/2));
			            tmp.points_y.push_back(floor(points_pair[jj].points_y[kk]/2));
                        tmp.valid_points_x.push_back(points_pair[jj].points_x[kk]);
			            tmp.valid_points_y.push_back(points_pair[jj].points_y[kk]);
					}
				}
			}

		    line_index.clear();
		    MatDbl tt_line_points;
		    tt_line_points.Reset(2,tmp.points_x.size());
		    for (int kkk=0; kkk<tmp.points_x.size(); kkk++)
			{
			    tt_line_points(0,kkk) = tmp.points_y[kkk];
			    tt_line_points(1,kkk) = tmp.points_x[kkk];
			}
		    LineFitting(tt_line_points, &line_index);
		    double slope;
		    slope = (-1.0*line_index[0])/line_index[1];
		    tmp.theta = 90.0 + atan(slope)*180.0/pi;
		    tmp.line_parameter = line_index;
		    tmp.id = ii;
		    int good_root;
		    BorderCheck(tmp, &good_root);
		    if( good_root == 1)
			{
				std::vector<SeedPoint> line_ends;
				FindTwoEnd(tmp, &line_ends);
                tmp.end_points = line_ends;
				root_points->push_back(tmp);
			}
		}
    }
}


void WriteMoreData(CString& file_name, const ImgGray& img_r, const ImgGray& img_g, const ImgGray& img_b, int img_size_x, int img_size_y, std::vector<int>& seed_rr_x, std::vector<int>& seed_rr_y)
{
	ImgGray img_a, img_bb, img_c;
	img_a.Reset(img_size_x,img_size_y);
	img_bb.Reset(img_size_x,img_size_y);
	img_c.Reset(img_size_x,img_size_y);
	img_a = img_r;
	img_bb = img_g;
	img_c = img_b;
	for ( int mm=0; mm<seed_rr_x.size(); mm++ )
	{
		int temp_x = seed_rr_x[mm];
		int temp_y = seed_rr_y[mm];
		for (int yy = temp_y - 2 ; yy <= temp_y + 2 ; yy++)
		{
			for (int xx = temp_x - 2 ; xx <= temp_x + 2 ; xx++)  
			{
				if (xx >= 0 && yy >= 0 && xx < img_size_x && yy < img_size_y)  
				{   
					int offset = yy * img_size_x + xx;
					*(img_a.Begin() + offset) = 255;
					*(img_bb.Begin() + offset) = 0;
					*(img_c.Begin() + offset) = 0;
				}
			}
		}
	}
	
	FILE *fp;
	if ( (fp = fopen(file_name, "wb")) == NULL)
		BLEPO_ERROR(StringEx("Can't open file named '%s' for writing\n", file_name));
	fprintf(fp, "P6\n");
	fprintf(fp, "%d %d\n", img_size_x, img_size_y);
	fprintf(fp, "255\n");

	unsigned char *b_ptr = img_a.Begin();
	unsigned char *g_ptr = img_bb.Begin();
	unsigned char *r_ptr = img_c.Begin();
	
	for (int jj = 0 ; jj < img_size_x; jj++)  
	{
		for (int ii = 0 ; ii < img_size_y; ii++)  
		{
			fwrite(r_ptr, 1, 1, fp); 
            fwrite(g_ptr, 1, 1, fp);
            fwrite(b_ptr, 1, 1, fp); 
			r_ptr++;
			g_ptr++;
			b_ptr++;
		}
	} 
	fclose(fp);

	unsigned char *bb_ptr = img_a.Begin();
	unsigned char *gg_ptr = img_bb.Begin();
	unsigned char *rr_ptr = img_c.Begin();
	ImgBgr root_img;
	root_img.Reset(img_size_x,img_size_y);
	unsigned char *ptri = root_img.BytePtr();
	int bb, gg, rr;
	for (int i=0 ; i<img_size_x*img_size_y; i++) 
	{
		    bb = *bb_ptr++;
		    gg = *gg_ptr++;
		    rr = *rr_ptr++;
		    *ptri++ = bb;
		    *ptri++ = gg;
		    *ptri++ = rr;
	} 
	


	ImgGray root_img_grey, root_img_grey_half;
	Convert(root_img, &root_img_grey);
	Downsample(root_img_grey, 2, 2, &root_img_grey_half);

	Figure fig_1;
    fig_1.Draw(root_img_grey_half);
}

void WriteData(CString& file_name, const ImgGray& img_r, const ImgGray& img_g, const ImgGray& img_b, int img_size_x, int img_size_y, std::vector<int>& seed_rr_x, std::vector<int>& seed_rr_y)
{
	ImgGray img_a, img_bb, img_c;
	img_a.Reset(img_size_x,img_size_y);
	img_bb.Reset(img_size_x,img_size_y);
	img_c.Reset(img_size_x,img_size_y);
	img_a = img_r;
	img_bb = img_g;
	img_c = img_b;
	for ( int mm=0; mm<seed_rr_x.size(); mm++ )
	{
		int temp_x = seed_rr_x[mm];
		int temp_y = seed_rr_y[mm];
		for (int yy = temp_y - 1 ; yy <= temp_y + 1 ; yy++)
		{
			for (int xx = temp_x - 1 ; xx <= temp_x + 1 ; xx++)  
			{
				if (xx >= 0 && yy >= 0 && xx < img_size_x && yy < img_size_y)  
				{   
					int offset = yy * img_size_x + xx;
					*(img_a.Begin() + offset) = 255;
					*(img_bb.Begin() + offset) = 0;
					*(img_c.Begin() + offset) = 0;
				}
			}
		}
	}
	
	FILE *fp;
	if ( (fp = fopen(file_name, "wb")) == NULL)
		BLEPO_ERROR(StringEx("Can't open file named '%s' for writing\n", file_name));
	fprintf(fp, "P6\n");
	fprintf(fp, "%d %d\n", img_size_x, img_size_y);
	fprintf(fp, "255\n");

	unsigned char *b_ptr = img_a.Begin();
	unsigned char *g_ptr = img_bb.Begin();
	unsigned char *r_ptr = img_c.Begin();

	
	for (int jj = 0 ; jj < img_size_x; jj++)  
	{
		for (int ii = 0 ; ii < img_size_y; ii++)  
		{
			fwrite(r_ptr, 1, 1, fp); 
            fwrite(g_ptr, 1, 1, fp);
            fwrite(b_ptr, 1, 1, fp); 
			r_ptr++;
			g_ptr++;
			b_ptr++;
		}
	} 
	fclose(fp);
}

void FindMaxima(const ImgFloat& img, int search_type, int skip_axis_val, double min_peak, double min_width, int scale, std::vector<SeedPoint>* maxima)
{
	SeedPoint tmp_seed;
	int local_min = -1;
	int ii = 1;
	int temp_max;
	double temp_val;
	int find_one = 0;
	while( ii < (img.Width()-1) )
	{
	    
		while( (img(ii) >= img(ii-1)) && (ii < (img.Width()-1)) )
		{
		    ii++;
		}
		if( img(ii-1) >= min_peak )
		{
			temp_max = ii-1;
			temp_val = img(ii-1);
			find_one = 1;
		}
		
        while ( (img(ii) <= img(ii-1)) && (ii < (img.Width()-1)) )
		{
		    ii++;
		}

		if( (local_min != -1) && ((ii-1 - local_min) >= min_width) && (find_one == 1) )
		{
			
			if(search_type == 0 )
			{
				tmp_seed.x = temp_max*scale;
			    tmp_seed.y = skip_axis_val;
			}
			if(search_type == 1 )
			{
				tmp_seed.x = skip_axis_val;
				tmp_seed.y = temp_max*scale; 
			}
			tmp_seed.width = (ii-1 - local_min)*scale;
			tmp_seed.val = temp_val;
            maxima->push_back(tmp_seed);
            find_one = 0;
		}
		local_min = ii-1;
        ii++;
	}
}

void SelectSeedPointsHorizontal(const ImgGray& sample_img, const ImgFloat& gauss_w, double skip_dist, double mountain_high, double mountain_wide, int scale, std::vector<SeedPoint>* hor_seed)
{
	double nnn = 0.0;
	ImgFloat img_single_row;
	ImgFloat img_single_row_smooth;
	int hor_search = 0;

	for(int nn=0; nn<sample_img.Height(); nn=nnn)
	{
		nnn = nnn+skip_dist;
		img_single_row.Reset(sample_img.Width(), 1);
		float *ptr4 = img_single_row.Begin();
        for ( int jj=0; jj<sample_img.Width(); jj++ )
		{   
			*(ptr4+jj) = sample_img(jj,nn);
		}
		Convolve(img_single_row, gauss_w, &img_single_row_smooth);
		
		int mod_val = (nn*scale)%(10);
		int current_val;
		if( mod_val != 0 )
		{
			current_val = nn*scale+2;	
		}
		else
		{
			current_val = nn*scale;	
		}

		FindMaxima(img_single_row_smooth, hor_search, current_val, mountain_high, mountain_wide, scale, hor_seed);
	}
}


void SelectSeedPointsVertical(const ImgGray& sample_img, const ImgFloat& gauss_h, double skip_dist, double mountain_high, double mountain_wide, int scale, std::vector<SeedPoint>* ver_seed)
{
	double nnn = 0.0;
	ImgFloat img_single_col;
	ImgFloat img_single_col_smooth;
	int ver_search = 1;
    
	for(int nn=0; nn<sample_img.Width(); nn=nnn)
	{
		nnn = nnn+skip_dist;
		img_single_col.Reset(1, sample_img.Height());
		float *ptr4 = img_single_col.Begin();
        for ( int jj=0; jj<sample_img.Height(); jj++ )
		{   
        	*(ptr4+jj) = sample_img(nn, jj);
		}
		Convolve(img_single_col, gauss_h, &img_single_col_smooth);
		int mod_val = (nn*scale)%(10);
		int current_val;
		if( mod_val != 0 )
		{
			current_val = nn*scale+2;	
		}
		else
		{
			current_val = nn*scale;	
		}
		ImgFloat img_single_tmp;
		img_single_tmp.Reset(sample_img.Height(), 1);
		ImgFloat::Iterator d = img_single_tmp.Begin();
        ImgFloat::ConstIterator s;
        for(s=img_single_col_smooth.Begin(); s!=img_single_col_smooth.End(); s++)
		{
			*d = *s;
            d++;
		}
		FindMaxima(img_single_tmp, ver_search, current_val, mountain_high, mountain_wide, scale, ver_seed);
	}

}


void SeedPointsCombineHorizontal(std::vector<SeedPoint>& horz_seed, int scale, std::vector<SeedPoint>* horz_total)
{
	if(scale == 0)
	{
		for(int ii =0; ii<horz_seed.size(); ii++)
		{
			SeedPoint tmp;
			tmp.x = horz_seed.at(ii).x;
			tmp.y = horz_seed.at(ii).y;
			tmp.width = horz_seed.at(ii).width;
			tmp.val = horz_seed.at(ii).val;
			horz_total->push_back(tmp);
		}
	}
	else
	{
		int dist_points;
	    for( int ii = 0; ii < horz_seed.size(); ii++)
		{
			for( int jj = 0; jj < horz_total->size(); jj++)
			{ 
				if( (horz_total->at(jj).y == horz_seed.at(ii).y) && (horz_total->at(jj).x != 9999) )
				{
					dist_points = floor(horz_seed.at(ii).width/2);
					if ( abs(horz_total->at(jj).x-horz_seed.at(ii).x) <= dist_points )
					{
						horz_total->at(jj).x = 9999;
					}
				} 
			}
		}
	  
		for(ii =0; ii<horz_seed.size(); ii++)
		{
			SeedPoint tmp;
			tmp.x = horz_seed.at(ii).x;
			tmp.y = horz_seed.at(ii).y;
			tmp.width = horz_seed.at(ii).width;
			tmp.val = horz_seed.at(ii).val;
			horz_total->push_back(tmp);
		}
		
	}
}

void SeedPointsCombineVertical(std::vector<SeedPoint>& vert_seed, int scale, std::vector<SeedPoint>* vert_total)
{
	if(scale == 0)
	{
	    for(int ii =0; ii<vert_seed.size(); ii++)
		{
			SeedPoint tmp;
			tmp.x = vert_seed.at(ii).x;
			tmp.y = vert_seed.at(ii).y;
			tmp.width = vert_seed.at(ii).width;
			tmp.val = vert_seed.at(ii).val;
			vert_total->push_back(tmp);
		}
	}
	else
	{
		int dist_points;
		for( int ii = 0; ii < vert_seed.size(); ii++)
		{
			for( int jj = 0; jj < vert_total->size(); jj++)
			{ 
				if( (vert_total->at(jj).x == vert_seed.at(ii).x) && (vert_total->at(jj).y != 9999) )
				{
					dist_points = floor(vert_seed.at(ii).width/2);
					if ( abs(vert_total->at(jj).y - vert_seed.at(ii).y) <= dist_points )
					{
						vert_total->at(jj).y = 9999;
					}
				} 
			}
		}

		for(ii =0; ii<vert_seed.size(); ii++)
		{
			SeedPoint tmp;
			tmp.x = vert_seed.at(ii).x;
			tmp.y = vert_seed.at(ii).y;
			tmp.width = vert_seed.at(ii).width;
			tmp.val = vert_seed.at(ii).val;
			vert_total->push_back(tmp);
		}
	}

}

void SeedPointSelection(const ImgGray& root_img, std::vector<SeedPoint>* hh_seed, std::vector<SeedPoint>* vv_seed)
{
	//const double grey_percentage = 0.7;
	const double grey_space = 2.5;
	const int step_space = 10;
	double grey_percentage;
	
	std::vector<SeedPoint> hor_seed;
	std::vector<SeedPoint> ver_seed;
	std::vector<SeedPoint> total_hor_seed;
	std::vector<SeedPoint> total_ver_seed;

	int sigma = 2; 
	int kernel_length = 9;
	ImgFloat gauss_x;
	ImgFloat gauss_y;
    Gauss(sigma,kernel_length,&gauss_x,&gauss_y);
    
    int new_seed_no = 0;
	double step_dist;
	double min_width;
	double min_peak;
	ImgGray grey_green_img;

    for(int pp = 0; pp<3; pp++)
	{
		hor_seed.clear();
		ver_seed.clear();
		int rr = pow(2, pp);
		Downsample(root_img, rr, rr, &grey_green_img);
		int max_val = Max(grey_green_img);
		if( max_val >= 204 )
		{
			grey_percentage = 0.7;
		}
		else
		{
			grey_percentage = 0.66;
		}
		min_peak = grey_percentage*max_val;
        step_dist = double(step_space)/(double)rr;
		min_width = grey_space*step_dist;
		SelectSeedPointsHorizontal(grey_green_img, gauss_x, step_dist, min_peak, min_width, rr, &hor_seed);
		SeedPointsCombineHorizontal(hor_seed, pp, &total_hor_seed);
		SelectSeedPointsVertical(grey_green_img, gauss_y, step_dist, min_peak, min_width, rr, &ver_seed);
		SeedPointsCombineVertical(ver_seed, pp, &total_ver_seed);
	}
	

	for(int ii=0; ii<total_hor_seed.size(); ii++)
	{
		if( total_hor_seed.at(ii).x != 9999)
		{
		    SeedPoint tmp;
			tmp.x = total_hor_seed.at(ii).x;
			tmp.y = total_hor_seed.at(ii).y;
			tmp.width = total_hor_seed.at(ii).width;
			tmp.val = total_hor_seed.at(ii).val;
			hh_seed->push_back(tmp);
		}
	}

	for(ii=0; ii<total_ver_seed.size(); ii++)
	{
		if( total_ver_seed.at(ii).y != 9999)
		{
		    SeedPoint tmp;
			tmp.x = total_ver_seed.at(ii).x;
			tmp.y = total_ver_seed.at(ii).y;
			tmp.width = total_ver_seed.at(ii).width;
			tmp.val = total_ver_seed.at(ii).val;
			vv_seed->push_back(tmp);
		}
	}
}

void RegionProps(ImgGray& tmp, RootElement* temp_refine)
{
	temp_refine->root_area = tmp;
	ImgBinary root_binary;
	root_binary.Reset(tmp.Width(), tmp.Height());
	Convert(tmp, &root_binary);
	RegionProperties *props_1 = new RegionProperties[1];
	RegionProps(root_binary, props_1);
	double root_theta;
	if( (props_1->direction)*180/pi >= 0 )
	{
		root_theta = 180 - (props_1->direction)*180/pi;
	}
	if( (props_1->direction)*180/pi < 0 )
	{
		root_theta = -1.0*(props_1->direction)*180/pi;
	}
    temp_refine->theta = root_theta;
	temp_refine->eccentricity = props_1->eccentricity;
	unsigned char *ptr = tmp.Begin();
	for (int ni=0 ; ni<tmp.Width()*tmp.Height(); ni++) 
	{
		if(*ptr++ ==255)
		{
			int rr = ni/(tmp.Width());
			int cc = ni%(tmp.Width());
			temp_refine->points_x.push_back(cc);
			temp_refine->points_y.push_back(rr);
		}
	}
}

void HistogramVal(ImgGray& img_half, RootElement& root1, int* hist_no)
{
	std::vector<int> hist_bin;
	hist_bin.clear();
	ImgGray root_hist;
	root_hist.Reset(img_half.Width(), img_half.Height());
	Set(&root_hist, 0);
	unsigned char *ptr = root1.root_area.Begin();
	for (int ni=0 ; ni<img_half.Width()*img_half.Height(); ni++) 
	{
		if(*ptr++ ==255)
		{
			int rr = ni/(img_half.Width());
			int cc = ni%(img_half.Height());
			root_hist(cc,rr) = img_half(cc, rr);
		}
	}
	HistogramGray(root_hist, 256, &hist_bin);
	int hist_sum = 0;
	for(int nnn =178; nnn<=255; nnn++)
	{
		hist_sum = hist_sum + hist_bin[nnn];
	}
	(*hist_no) = hist_sum;
}

void EdgeVal(ImgGray& img_half, RootElement& root1, int* edge_no)
{
	int ImgSizeX = img_half.Width();
    int ImgSizeY = img_half.Height();

	ImgGray root_edge;
	root_edge.Reset(ImgSizeX, ImgSizeY);
	Sobel(img_half, &root_edge);
	unsigned char K;
	Otsu(root_edge, &K);
	ImgBinary root_edge_binary;
	root_edge_binary.Reset(ImgSizeX, ImgSizeY);
	Threshold(root_edge, K, &root_edge_binary);
	ImgGray root_edge_gray;
	root_edge_gray.Reset(ImgSizeX, ImgSizeY);
	Convert(root_edge_binary, &root_edge_gray);
	And(root1.root_area, root_edge_gray, &root_edge_gray);
	ImgGray root_boundary_1, root_boundary_2;
	root_boundary_1.Reset(ImgSizeX, ImgSizeY);
	root_boundary_2.Reset(ImgSizeX, ImgSizeY);
	Erode3x3(root1.root_area, &root_boundary_1);
	Subtract(root1.root_area, root_boundary_1, &root_boundary_2);
					
	ImgGray true_edge;
	true_edge.Reset(ImgSizeX, ImgSizeY);
	Set(&true_edge, 0);
	int edge_points_no = 0;
	int cou;
	for(int cc = 0; cc< ImgSizeX; cc++)
	{
		for(int rr = 0; rr< ImgSizeY; rr++)
		{
			cou = 0;
			if( root_edge_gray(cc,rr) != 0)
			{
				for (int qq=-1; qq<=1; qq++)
				{
					for (int qqq=-1; qqq<=1; qqq++)
					{ 
						if(  ((cc+qq)>=0) && ((rr+qqq)>=0) && ((cc+qq)<root_boundary_2.Width()) && ((rr+qqq)<root_boundary_2.Height()) )
						{
							if( root_boundary_2(cc+qq, rr+qqq) == 255) 
							{
								cou = cou + 1;
							}
						}
					}
				}
				if( cou == 0 )
				{
					edge_points_no = edge_points_no + 1;
					true_edge(cc,rr) = 255;
				}
			}
		}
	}
	(*edge_no) = edge_points_no;
}

void FindFeatures(ImgGray& RootArea, std::vector<SeedPoint>& centeraxis, double* root_diameter, int* als_no, int* pb_no)
{
	ImgGray root_half;
	root_half = RootArea;
	std::vector<double> centeraxis_theta;
	centeraxis_theta.clear();
	if ( centeraxis.size() < 10 )
	{
		double theta_sum;
		double theta_mean;
		int ver_slope, hor_slope, avg_slope;
		theta_sum = 0.0;
		theta_mean = 0.0;
		ver_slope = 0;
		hor_slope = 0;
        avg_slope = 0;
		int loc_middle = floor(centeraxis.size()/2);
		for ( int nnn = 0; nnn<centeraxis.size(); nnn++ )
		{
		    int coordinate_x, coordinate_y;
			double theta_slope;
			theta_slope = 0.0;
            coordinate_x = 0;
			coordinate_y = 0;
            coordinate_x = centeraxis[nnn].x - centeraxis[loc_middle].x;
			coordinate_y = centeraxis[nnn].y - centeraxis[loc_middle].y;
			if( (coordinate_x != 0) && (coordinate_y != 0) )
			{
				theta_slope = double(coordinate_y)/double(coordinate_x);
				theta_sum = theta_sum+theta_slope;
				avg_slope = avg_slope + 1;
			}
			if( (coordinate_x == 0) && (coordinate_y != 0) )
			{
				ver_slope = ver_slope + 1;
			}
			if( (coordinate_x != 0) && (coordinate_y == 0) )
			{
				hor_slope = hor_slope + 1;
			}
		}
		if( hor_slope >= floor(centeraxis.size()/2) )
		{
			theta_mean = 0.01;
		}
		else if( ver_slope >= floor(centeraxis.size()/2) )
		{
			theta_mean = 99999.0;
		}
		else
		{
			theta_mean = theta_sum/avg_slope;
		}
		for(nnn=0; nnn<centeraxis.size(); nnn++)
		{
			centeraxis_theta.push_back(theta_mean);
		}
	}

	if ( centeraxis.size() >= 10 )
	{
		for( int nnn=4; nnn < (centeraxis.size()-5); nnn++)
		{
			double theta_sum;
			double theta_mean;
			int ver_slope, hor_slope, avg_slope;
			theta_sum = 0.0;
			theta_mean = 0.0;
			ver_slope = 0;
			hor_slope = 0;
			avg_slope = 0;
			for(int mmm = -4; mmm<=5; mmm++)
			{
				int coordinate_x, coordinate_y;
				double theta_slope;
				theta_slope = 0.0;
                coordinate_x = 0;
				coordinate_y = 0;
				ver_slope = 0;
				hor_slope = 0;
                coordinate_x = centeraxis[nnn+mmm].x - centeraxis[nnn].x;
				coordinate_y = centeraxis[nnn+mmm].y - centeraxis[nnn].y;
				if( (coordinate_x != 0) && (coordinate_y != 0) )
				{
					theta_slope = double(coordinate_y)/double(coordinate_x);
					theta_sum = theta_sum+theta_slope;
					avg_slope = avg_slope + 1;
				}
				if( (coordinate_x == 0) && (coordinate_y != 0) )
				{
					ver_slope = ver_slope + 1;
				}
				if( (coordinate_x != 0) && (coordinate_y == 0) )
				{
					hor_slope = hor_slope + 1;
				}
			}
			if( hor_slope >= 4 )
			{
				theta_mean = 0.01;
			}
			else if( ver_slope >= 4 )
			{
				theta_mean = 99999.0;
			}
			else
			{
				theta_mean = theta_sum/avg_slope;
			}
            centeraxis_theta.push_back(theta_mean);
		}
		std::vector<double>::iterator it = centeraxis_theta.begin();
		double insert_val = centeraxis_theta[0];
        centeraxis_theta.insert(it, 4, insert_val);
		it = centeraxis_theta.end();
		insert_val = centeraxis_theta[centeraxis_theta.size()-1];
		centeraxis_theta.insert(it, 4, insert_val);
	}
    std::vector<double> centeraxis_perpendicular;
	centeraxis_perpendicular.clear();
	for(int nnn = 0; nnn<centeraxis_theta.size(); nnn++)
	{
		centeraxis_perpendicular.push_back(-1.0/centeraxis_theta[nnn]);
	}
	
	ImgGray root_border, temp_border;
	root_border.Reset(root_half.Width(), root_half.Height());
	temp_border.Reset(root_half.Width(), root_half.Height());
	Erode3x3(root_half, &temp_border);
    Subtract(root_half, temp_border, &root_border);
    
	std::vector<SeedPoint> border;
	border.clear();
	unsigned char *ptr = root_border.Begin();
	for (int ni=0 ; ni<root_half.Width()*root_half.Height(); ni++) 
	{
		if(*ptr++ == 255)
		{
			int rr = ni/(root_half.Width());
			int cc = ni%(root_half.Width());
			SeedPoint tmp;
			tmp.x = cc;
			tmp.y = rr;
			border.push_back(tmp);
		}
	}
	
	double temp_diameter;
    int temp_y, temp_yy;
	std::vector<SeedPoint> border_up;
	std::vector<SeedPoint> border_down;
	std::vector<int> diameter_val;
    
	nnn = 0;
	int good_no = 0;
	int good_bound = 0;
	while( nnn<centeraxis.size() )
	{
		border_up.clear();
		border_down.clear();

		for(int mmm = 0; mmm<border.size(); mmm++)
		{
			if( (centeraxis[nnn].x!=border[mmm].x) && (centeraxis[nnn].y!=border[mmm].y) )
			{
				temp_y = floor(centeraxis_perpendicular[nnn]*(border[mmm].x-centeraxis[nnn].x))+centeraxis[nnn].y-border[mmm].y;
				if( abs(temp_y) <= 1 )
				{
					temp_yy = floor(centeraxis_theta[nnn]*(border[mmm].x-centeraxis[nnn].x))+centeraxis[nnn].y-border[mmm].y;
					if( temp_yy > 0 )
					{
						SeedPoint tmp;
						tmp.x = border[mmm].x;
						tmp.y = border[mmm].y;
						border_up.push_back(tmp);
					}
					if( temp_yy < 0 )
					{
						SeedPoint tmp;
						tmp.x = border[mmm].x;
						tmp.y = border[mmm].y;
						border_down.push_back(tmp);
					}
				}
			}
		}
        
		int neighbor_angle[5][5];
		neighbor_angle[0][0]=135; neighbor_angle[0][1]=120; neighbor_angle[0][2]=90; neighbor_angle[0][3]=60;   neighbor_angle[0][4]=45;
		neighbor_angle[1][0]=150; neighbor_angle[1][1]=135; neighbor_angle[1][2]=90; neighbor_angle[1][3]=45;   neighbor_angle[1][4]=30;
		neighbor_angle[2][0]=180; neighbor_angle[2][1]=180; neighbor_angle[2][2]=0;  neighbor_angle[2][3]=0;    neighbor_angle[2][4]=0;
		neighbor_angle[3][0]=30;  neighbor_angle[3][1]=45;  neighbor_angle[3][2]=90; neighbor_angle[3][3]=135;  neighbor_angle[3][4]=150;
		neighbor_angle[4][0]=45;  neighbor_angle[4][1]=60;  neighbor_angle[4][2]=90; neighbor_angle[4][3]=120;  neighbor_angle[4][4]=135;
        
		int neighbor_angle_up;
	    int neighbor_angle_down;
		double avg_angle_up;
	    double avg_angle_down;
		double side_dist_1, side_dist_2, side_dist_ratio;
		if( (border_up.size()>0) && (border_down.size()>0) )
		{
			temp_diameter = sqrt(pow((border_down[0].x-border_up[0].x), 2)+pow((border_up[0].y-border_down[0].y), 2));
			diameter_val.push_back(temp_diameter);
			neighbor_angle_down = 0;
			int cc = border_down[0].x;
			int rr = border_down[0].y;
			int white_no = 0;
            for (int s=0; s<=4; s++)
			{
				for (int ss=0; ss<=4; ss++)
				{
					if( ((cc+s-2)>=0) && ((cc+s-2)<root_border.Width()) && ((rr+ss-2)>=0) && ((rr+ss-2)<root_border.Height()))
					{
						if( root_border(cc+s-2, rr+ss-2) == 255 )
						{
						    neighbor_angle_down = neighbor_angle_down + neighbor_angle[s][ss];
						    white_no = white_no + 1;
						}
					}
				}
			}
			avg_angle_down = double(neighbor_angle_down)/double(white_no);
			neighbor_angle_up = 0;
			cc = border_up[0].x;
			rr = border_up[0].y;
			white_no = 0;
            for (s=0; s<=4; s++)
			{
				for (int ss=0; ss<=4; ss++)
				{
					if( ((cc+s-2)>=0) && ((cc+s-2)<root_border.Width()) && ((rr+ss-2)>=0) && ((rr+ss-2)<root_border.Height()))
					{
						if( root_border(cc+s-2, rr+ss-2) == 255 ) 
						{
						    neighbor_angle_up = neighbor_angle_up + neighbor_angle[s][ss];
						    white_no = white_no + 1;
						}
					}
				}
			}
		    avg_angle_up = double(neighbor_angle_up)/double(white_no);
			if( abs(avg_angle_down-avg_angle_up)<=25 )
			{
				good_bound = good_bound+1;
			}
			
			side_dist_1 = sqrt(pow((border_down[0].x-centeraxis[nnn].x), 2)+pow((border_down[0].y-centeraxis[nnn].y), 2));
			side_dist_2 = sqrt(pow((border_up[0].x-centeraxis[nnn].x), 2)+pow((border_up[0].y-centeraxis[nnn].y), 2));
			side_dist_ratio = side_dist_1/side_dist_2;
			if ( (side_dist_ratio>=0.75) && (side_dist_ratio<=1.33) )
			{
				good_no = good_no + 1;
			}
		}
		nnn = nnn + 1;	
	}
	*(als_no) = good_no;
	*(pb_no) = good_bound;
	if( diameter_val.size() > 0 )
	{
		int median_loc = 0;
	    double median_val = 0.0;
        sort(diameter_val.begin(), diameter_val.end());
	    median_loc = floor(diameter_val.size()/2);
	    if( 2*median_loc != diameter_val.size() )
		{
		    median_val = diameter_val[median_loc];
		}
	    else
		{
		    median_val = (diameter_val[median_loc]+diameter_val[median_loc-1])/2;
		}
        *(root_diameter) = median_val;
	}
	else
	{
		*(root_diameter) = 3.0;
	}
}

void RootsRegionRefine(ImgGray& img_half, std::vector<RootElement>&grown_roots, std::vector<RootElement>* refine_roots)
{
	EquivalenceTable equiv_table;
	std::vector<SeedPoint> overlap_points;
	int img_width = grown_roots[0].root_area.Width();
	int img_height = grown_roots[0].root_area.Height();
    
	std::vector<SeedPoint> centerline_points;
	double root_diameter;
	int pb_no, als_no;
	for(int kk=0; kk<grown_roots.size(); kk++)
	{
		centerline_points.clear();
		FindLengthPath(grown_roots[kk].root_area, grown_roots[kk].theta, &centerline_points);
		FindFeatures(grown_roots[kk].root_area, centerline_points, &root_diameter, &als_no, &pb_no);
		grown_roots[kk].width = root_diameter;
	}
    
	for(kk=0; kk<grown_roots.size()-1; kk++)
	{
		for(int kkk=kk+1; kkk<grown_roots.size(); kkk++)
		{
			if( grown_roots[kk].id == grown_roots[kkk].id)
			{
				equiv_table.AddEquivalence(kk, kkk);
			}
			else
			{
				ImgGray root_check;
				root_check.Reset(img_width, img_height);
			    Set(&root_check, 0);
			    And(grown_roots[kk].root_area, grown_roots[kkk].root_area, &root_check);
			    overlap_points.clear();
			    unsigned char *ptr = root_check.Begin();
		    	for (int ni=0 ; ni<img_width*img_height; ni++) 
				{
				    if(*ptr++ ==255)
					{
					    int rr = ni/(img_width);
					    int cc = ni%(img_width);
					    SeedPoint tmp;
					    tmp.x = cc;
				    	tmp.y = rr;
				    	overlap_points.push_back(tmp);
					}
				}
				
				int small_size, large_size;
			    double overlap_ratio;
			    if( overlap_points.size() > 0 )
				{   
				    if(grown_roots[kk].points_x.size() >= grown_roots[kkk].points_x.size())
					{
					    large_size = kk;
					    small_size = kkk;
				    	overlap_ratio = double(overlap_points.size())/double(grown_roots[kkk].points_x.size());
					}
				    else
					{
				    	small_size = kk;
				    	large_size = kkk;
				    	overlap_ratio = double(overlap_points.size())/double(grown_roots[kk].points_x.size());
					}
				    double orient_diff = abs(grown_roots[kk].theta-grown_roots[kkk].theta);
				    double width_diff;
				    if(grown_roots[kk].width>grown_roots[kkk].width)
					{
				        width_diff = grown_roots[kk].width/grown_roots[kkk].width;
					}
				    else
					{
				    	width_diff = grown_roots[kkk].width/grown_roots[kk].width;
					}
                    //printf("%d %d %5.3f %5.3f %5.3f %5.3f\r\n", kk, kkk, overlap_ratio, orient_diff, grown_roots[kkk].width, grown_roots[kk].width);
				    if( overlap_ratio >= 0.6)
					{
					    equiv_table.AddEquivalence(kk, kkk);
					}
				    else if ( (overlap_ratio < 0.6) && (overlap_ratio >= 0.2) && (width_diff < 2.0) && ( (orient_diff >= 100.0) || (orient_diff <= 6.0) ) )
					{
					    equiv_table.AddEquivalence(kk, kkk);	
					}
				    else
					{
					    equiv_table.AddEquivalence(kk, kk);
					    equiv_table.AddEquivalence(kkk, kkk);
					}
				}
			    else
				{
				    equiv_table.AddEquivalence(kk, kk);
				    equiv_table.AddEquivalence(kkk, kkk);
				}
			}
		}
	}
    equiv_table.TraverseLinks();
    int label_no = equiv_table.RemoveGaps();
    for (int ii=0; ii<grown_roots.size(); ii++)
	{ 
		grown_roots[ii].id = equiv_table.GetEquivalentLabel(ii);
	}
    
	
    std::vector<int> temp_points_x;
	std::vector<int> temp_points_y;
	std::vector<SeedPoint> total_end;
	ImgGray tmp;
    tmp.Reset(img_width, img_height);
	for(ii=0; ii<label_no; ii++)
	{
		temp_points_x.clear();
		temp_points_y.clear();
		total_end.clear();
		Set(&tmp, 0);
		for(int jj=0; jj<grown_roots.size(); jj++)
		{
			if( grown_roots[jj].id == ii )
			{
				Or(tmp, grown_roots[jj].root_area, &tmp);
				for(int tt=0; tt<grown_roots[jj].valid_points_x.size(); tt++)
				{
					temp_points_x.push_back(grown_roots[jj].valid_points_x[tt]);
					temp_points_y.push_back(grown_roots[jj].valid_points_y[tt]);
				}
				
				SeedPoint tmp_end;
				for (int xxx = 0; xxx < grown_roots[jj].end_points.size(); xxx++)
				{
					tmp_end.x = grown_roots[jj].end_points[xxx].x;
					tmp_end.y = grown_roots[jj].end_points[xxx].y;
					total_end.push_back(tmp_end);
				}
			}
		}
		RootElement temp_refine;
		temp_refine.end_points = total_end;
		temp_refine.valid_points_x = temp_points_x;
		temp_refine.valid_points_y = temp_points_y;
		RegionProps(tmp, &temp_refine);

        std::vector<SeedPoint> CenterlinePoints;
		FindLengthPath(tmp, temp_refine.theta, &CenterlinePoints);
		for( int xxx = 0; xxx < CenterlinePoints.size(); xxx++)
		{
			temp_refine.centeraxis_points_x.push_back(CenterlinePoints[xxx].x);
			temp_refine.centeraxis_points_y.push_back(CenterlinePoints[xxx].y);
		}

		refine_roots->push_back(temp_refine);
	}
}

void RootClassify(ImgGray& img_half, RootElement& region, int* root_true)
{
	ImgInt img_lables;
	ConnectedComponents(region.root_area, &img_lables);
	int ImgWidth = region.root_area.Width();
	int ImgHeight = region.root_area.Height();
	int componet_no = Max(img_lables);
	std::vector<SeedPoint> centerline_points;
    //printf("%d \r\n", componet_no);
	int pb_sum = 0;
	int als_sum = 0;
	int ht_sum = 0;
	int length_sum = 0;
	int edge_sum = 0;
	double diam_sum = 0.0;
	int part_no = 0;
	double eccent_val = 0.0;
	int area_sum = 0;

	for(int xxx =1; xxx<=componet_no; xxx++)
	{
		ImgGray new_root;
		new_root.Reset(ImgWidth, ImgHeight);
		Set(&new_root, 0);
		unsigned char *ptr1 = new_root.Begin();
		int *ptr = img_lables.Begin();
		for (int ni=0 ; ni<ImgWidth*ImgHeight; ni++) 
		{
			if(*ptr++ == xxx)
			{
				*ptr1 = 255;
			}
			*ptr1++;
		}
		RootElement root_part;
		RegionProps(new_root, &root_part);
		//printf("%d \r\n", root_part.points_x.size());
		if(root_part.points_x.size() >= 40)
		{
			int pb_no = 0;
			int als_no = 0;
			int ht_no = 0;
			int edge_no = 0;
			double root_diameter = 0.0;

			centerline_points.clear();
		    FindLengthPath(root_part.root_area, root_part.theta, &centerline_points);
	    	FindFeatures(root_part.root_area, centerline_points, &root_diameter, &als_no, &pb_no);
		    HistogramVal(img_half, root_part, &ht_no);
			EdgeVal(img_half, root_part, &edge_no);
			length_sum = length_sum + centerline_points.size();
			als_sum = als_sum + als_no;
			ht_sum = ht_sum + ht_no;
			pb_sum = pb_sum + pb_no;
			edge_sum = edge_sum + edge_no;
			diam_sum = diam_sum + root_diameter;
			eccent_val = eccent_val + root_part.eccentricity;
			part_no = part_no + 1;
			area_sum = area_sum + root_part.points_x.size();
			//printf("%d %d %d %d %5.3f %d \r\n", length_sum, als_sum, ht_sum, pb_sum, diam_sum, part_no);
		}	
	}
	region.eccentricity = eccent_val;
    region.ratio_pb = double(pb_sum)/double(length_sum);
	region.ratio_als = double(als_sum)/double(length_sum);
	region.ratio_hist = double(ht_sum)/double(area_sum);
	region.ratio_edge = 1.0 - double(edge_sum)/double(area_sum);
	region.width = double(diam_sum)/double(part_no);
	//printf("%5.3f %5.3f %5.3f %5.3f %5.3f %5.3f \r\n", region.ratio_edge, region.eccentricity, region.ratio_pb, region.ratio_als, region.ratio_hist, region.width);

	double EccentricityThresh = 5.28;
	double EdgeThresh = 0.9563;
	double HistogramThresh = 0.1764;
	double PbThresh = 0.4119;
	double AlsThresh = 0.9699;
	double EccentricityCoef = 1.10;
	double EdgeCoef = 0.16;
	double HistogramCoef = 0.63;
	double PbCoef = 0.78;
	double AlsCoef = 0.56;

	double ClassifyResult = 0.0;
	double ClassifyThresh = 0.084;
	int RootTrue = 0;
	if( region.eccentricity >= EccentricityThresh)
	{
		ClassifyResult = ClassifyResult + EccentricityCoef;
	}
	else
	{
		ClassifyResult = ClassifyResult - EccentricityCoef;
	}
	if( region.ratio_hist >= HistogramThresh)
	{
		ClassifyResult = ClassifyResult + HistogramCoef;
	}
	else
	{
		ClassifyResult = ClassifyResult - HistogramCoef;
	}
	if( region.ratio_edge >= EdgeThresh)
	{
		ClassifyResult = ClassifyResult + EdgeCoef;
	}
	else
	{
		ClassifyResult = ClassifyResult - EdgeCoef;
	}
	if( region.ratio_als >= PbThresh)
	{
		ClassifyResult = ClassifyResult + PbCoef;
	}
	else
	{
		ClassifyResult = ClassifyResult - PbCoef;
	}
	if( region.ratio_pb >= AlsThresh)
	{
		ClassifyResult = ClassifyResult + AlsCoef;
	}
	else
	{
		ClassifyResult = ClassifyResult - AlsCoef;
	}
	
	if(ClassifyResult >= ClassifyThresh)
	{
		RootTrue = 1;
	}
	else
	{
		RootTrue = 0;
	}
	(*root_true) = RootTrue;
	//printf("%d \r\n", RootTrue);
}


void DetectRoots(const blepo::ImgBgr& img_load, std::vector<DetectedRoot>* output)
{
	// can call anything you want
//	ImgGray img_1, img_2, img_3;
	ImgGray img_2;
	const unsigned char *ptri = img_load.BytePtr();
    const int ImgSizeX = img_load.Width(), ImgSizeY = img_load.Height();
//	img_1.Reset(ImgSizeX,ImgSizeY);
	img_2.Reset(ImgSizeX,ImgSizeY);
//	img_3.Reset(ImgSizeX,ImgSizeY);
 
//	unsigned char *ptr1 = img_1.Begin();
	unsigned char *ptr2 = img_2.Begin();
//	unsigned char *ptr3 = img_3.Begin();
//	int b, g ,r, i;
	int i;
		
	for (i=ImgSizeX*ImgSizeY ; i>0 ; i--) 
	{
    ptri++;
    *ptr2++ = *ptri++;
    ptri++;
//		b = *ptri++;
//		g = *ptri++;
//		r = *ptri++;
//		*ptr1++ = b;
//		*ptr2++ = g;
//		*ptr3++ = r;
	} 

	///////////// Extract green component End //
	std::vector<SeedPoint> SeedHor;
	std::vector<SeedPoint> SeedVer;
	std::vector<SeedPoint> SeedClean;
	SeedPointSelection(img_2, &SeedHor, &SeedVer);
	SeedClean = SeedVer;
        
	for(int ii = 0; ii<SeedHor.size(); ii++)
	{
		int find_one = 0;
		for(int jj = 0; jj<SeedClean.size(); jj++)
		{
			if( (SeedHor[ii].x == SeedClean[jj].x) && (SeedHor[ii].y == SeedClean[jj].y) )
			{
				if(SeedHor[ii].width <= SeedClean[jj].width)
				{
					SeedClean[jj].width = SeedHor[ii].width;
				}
				find_one = 1;
			}
		}
		if( find_one == 0 )
		{
			SeedClean.push_back(SeedHor[ii]);
		}
	}
		
	if( SeedClean.size() <= 500)
	{
		std::vector<RootElement> RootRow;
		std::vector<RootElement> RootCol;
		std::vector<RootElement> RootTotal;

        RootSelect(SeedHor, &RootRow);
		if(RootRow.size() > 0)
		{
			for(int kkk = 0; kkk<RootRow.size(); kkk++)
			{
				RootElement tmp;
			    tmp.line_parameter = RootRow[kkk].line_parameter;
			    tmp.length = RootRow[kkk].length;
			    tmp.theta = RootRow[kkk].theta;
		    	tmp.points_x = RootRow[kkk].points_x;
		    	tmp.points_y = RootRow[kkk].points_y;
		    	RootTotal.push_back(tmp);
			} 
		}

		RootSelect(SeedVer, &RootCol);
		if(RootCol.size() > 0)
		{
			for(int kkk = 0; kkk<RootCol.size(); kkk++)
			{
				RootElement tmp;
			    tmp.line_parameter = RootCol[kkk].line_parameter;
			    tmp.length = RootCol[kkk].length;
			    tmp.theta = RootCol[kkk].theta;
			    tmp.points_x = RootCol[kkk].points_x;
		        tmp.points_y = RootCol[kkk].points_y;
		        RootTotal.push_back(tmp);
			} 
		}
		
		std::vector<RootElement> CleanRoots;
		if( RootTotal.size() > 0)
		{
			if( RootTotal.size() >= 1)
			{
				RootsClean(RootTotal, SeedClean, &CleanRoots);
			}
			std::vector<RootElement> CombinedRoots;
		    if( CleanRoots.size() > 1 )
			{
				RootsCombine(CleanRoots, &CombinedRoots);
			    //for(int kkk = 0; kkk<CombinedRoots.size(); kkk++)
				//{
					//ImgGray img1_half, img2_half, img3_half;
		            //Downsample(img_1, 2, 2, &img1_half);
		            //Downsample(img_2, 2, 2, &img2_half);
		            //Downsample(img_3, 2, 2, &img3_half);
			        //CString file_line;
		    	    //file_line.Format("img%d_combined_root_%d.ppm", ff, kkk);
			        //WriteData(file_line, img1_half, img2_half, img3_half, ImgSizeX/2, ImgSizeY/2, CombinedRoots[kkk].points_x, CombinedRoots[kkk].points_y);
				 //}
			}
		    if( CleanRoots.size() == 1 )
			{
				RootElement tmp;
			    tmp = CleanRoots[0];
			    LineParameter temp;
			    temp.a = CleanRoots[0].line_parameter[0];
			    temp.b = CleanRoots[0].line_parameter[1];
			    temp.c = CleanRoots[0].line_parameter[2];
			    temp.d = CleanRoots[0].line_parameter[3];
			    tmp.total_line_para.push_back(temp);
			    tmp.points_no.push_back(CleanRoots[0].points_x.size());
				tmp.end_points = CleanRoots[0].end_points;
			    CombinedRoots.push_back(tmp);
			
			    //ImgGray img1_half, img2_half, img3_half;
		        //Downsample(img_1, 2, 2, &img1_half);
		        //Downsample(img_2, 2, 2, &img2_half);
		        //Downsample(img_3, 2, 2, &img3_half);
			    //CString file_line;
			    //file_line.Format("img%d_combined_root_%d.ppm", ff, 0);
			    //WriteData(file_line, img1_half, img2_half, img3_half, ImgSizeX/2, ImgSizeY/2, CombinedRoots[0].points_x, CombinedRoots[0].points_y);
			}
			
			ImgGray img2_half;
			Downsample(img_2, 2, 2, &img2_half);
			std::vector<SeedPoint> CenterlinePoints;
			std::vector<RootElement> GrownRoots;
		    RootsRegionGrowing(CombinedRoots, img_2, &GrownRoots);
            
            std::vector<RootElement> RefineRoots;
            if(GrownRoots.size() > 1)
			{
				RootsRegionRefine(img2_half, GrownRoots, &RefineRoots);
			}
			if(GrownRoots.size() == 1)
			{
				RootElement tmp;
			    RegionProps(GrownRoots[0].root_area, &tmp);
				tmp.end_points = GrownRoots[0].end_points;
                tmp.valid_points_x = GrownRoots[0].valid_points_x;
				tmp.valid_points_y = GrownRoots[0].valid_points_y;
			    CenterlinePoints.clear();
			    double root_diameter;
			    int pb_no, als_no;
			    FindLengthPath(tmp.root_area, tmp.theta, &CenterlinePoints);
				for( int xxx = 0; xxx < CenterlinePoints.size(); xxx++)
				{
					tmp.centeraxis_points_x.push_back(CenterlinePoints[xxx].x);
					tmp.centeraxis_points_y.push_back(CenterlinePoints[xxx].y);
				}
		        FindFeatures(tmp.root_area, CenterlinePoints, &root_diameter, &pb_no, &als_no);
			    tmp.width = root_diameter;
			    RefineRoots.push_back(tmp);
			}

			//FILE *fpt;
			//CString classifier_data;
			//classifier_data.Format("img%d_classify.txt", ff);
			//fpt=fopen(classifier_data,"wb");
			if( RefineRoots.size() > 0 )
			{
				printf("%d \r\n", RefineRoots.size());
				//fprintf(fpt, "%d \r\n", RefineRoots->size());
				for(int kkk = 0; kkk<RefineRoots.size(); kkk++)
				{ 
					int root_true = 0;
					if( (RefineRoots[kkk].valid_points_x.size() >= 15) && (RefineRoots[kkk].points_x.size() >= 8000) )
					{
						root_true = 0;
					}
					else if( (RefineRoots[kkk].valid_points_x.size() >= 15) && (RefineRoots[kkk].points_x.size() < 8000) )
					{
						root_true = 1;
					}
					else if( (RefineRoots[kkk].valid_points_x.size()) <= 5)
					{
						root_true = 0;
					}
					else
					{
				        root_true = 0;
						RootClassify(img2_half, RefineRoots[kkk], &root_true);
					}

					if( root_true == 1 )
					{
						
						DetectedRoot tmp;
						tmp.diameter = RefineRoots[kkk].width*2.0;
						//printf("%d %d\r\n", RefineRoots[kkk].points_x.size(), RefineRoots[kkk].centeraxis_points_x.size());
						//for( int xxx = 0; xxx<RefineRoots[kkk].valid_points_x.size(); xxx++)
						//{
							//RootPoint temp;
							//temp.x = RefineRoots[kkk].valid_points_x[xxx];
							//temp.y = RefineRoots[kkk].valid_points_y[xxx];
							//tmp.points.push_back(temp);
						//}
						//tmp.points.push_back(temp_points);
                     
						//// Save every 10th points on the center curve of root, and then plot it ////
						for( int xxx = 0; xxx<RefineRoots[kkk].centeraxis_points_x.size(); xxx=xxx+10)
						{
							RootPoint temp;
							temp.x = RefineRoots[kkk].centeraxis_points_x[xxx]*2;
							temp.y = RefineRoots[kkk].centeraxis_points_y[xxx]*2;
							tmp.points.push_back(temp);
						}
						
                        
						//// Sort the end points before plot them ////
						
						/*std::vector<int> point_x;
						std::vector<int> point_y;
						std::vector<int> temp_x;
						std::vector<int> temp_y;
						double root_orient = RefineRoots[kkk].theta;
						//printf("%d \r\n", RefineRoots[kkk].end_points.size());
						if( (root_orient >= 45.0) && (root_orient <= 135.0) )
						{
							for( int xxx = 0; xxx<RefineRoots[kkk].end_points.size(); xxx++)
							{
								temp_x.push_back(RefineRoots[kkk].end_points[xxx].x);
								temp_y.push_back(RefineRoots[kkk].end_points[xxx].y);
								//RootPoint temp;
								//temp.x = RefineRoots[kkk].end_points[xxx].x;
								//temp.y = RefineRoots[kkk].end_points[xxx].y;
								//tmp.points.push_back(temp);
							}
							point_y = temp_y;
							sort(point_y.begin(), point_y.end());
							for(xxx=0; xxx<point_y.size(); xxx++)
							{
								for(int yyy=0; yyy<temp_y.size(); yyy++)
								{
									if( point_y[xxx] == temp_y[yyy] )
									{
										point_x.push_back(temp_x[yyy]);
									}
								}
								RootPoint temp;
								temp.x = point_x[xxx];
								temp.y = point_y[xxx];
								tmp.points.push_back(temp);
							}

						}
						else
						{
							for( int xxx = 0; xxx<RefineRoots[kkk].end_points.size(); xxx++)
							{
								temp_x.push_back(RefineRoots[kkk].end_points[xxx].x);
								temp_y.push_back(RefineRoots[kkk].end_points[xxx].y);
								//RootPoint temp;
								//temp.x = RefineRoots[kkk].end_points[xxx].x;
								//temp.y = RefineRoots[kkk].end_points[xxx].y;
								//tmp.points.push_back(temp);
							}
							point_x = temp_x;
							sort(point_x.begin(), point_x.end());
							for(xxx=0; xxx<point_x.size(); xxx++)
							{
								for(int yyy=0; yyy<temp_x.size(); yyy++)
								{
									if( point_x[xxx] == temp_x[yyy] )
									{
										point_y.push_back(temp_y[yyy]);
									}
								}
								RootPoint temp;
								temp.x = point_x[xxx];
								temp.y = point_y[xxx];
								tmp.points.push_back(temp);
							}
						}*/

						output->push_back(tmp);
					}
			        //fprintf(fpt, "%5.3f %5.3f %5.3f %5.3f %5.3f\r\n", RefineRoots->at(kkk).ratio_hist, RefineRoots->at(kkk).ratio_edge, RefineRoots->at(kkk).eccentricity, RefineRoots->at(kkk).ratio_als, RefineRoots->at(kkk).ratio_pb);
			        //fprintf(fpt, "%d \r\n", root_true);
 
			    	//CString file_line;
			    	//file_line.Format("img%d_refine_root_%d.ppm", 1, kkk);
			    	//Save(RefineRoots[kkk].root_area, file_line, "ppm");
					
					//CString file_line;
		    	    //file_line.Format("img%d_combined_root_%d.ppm", 3, kkk);
			        //WriteData(file_line, img_1, img_2, img_3, ImgSizeX, ImgSizeY, RefineRoots[kkk].valid_points_x, RefineRoots[kkk].valid_points_y);
				}
			}
		    //fclose(fpt);
		}
    }
}



