#ifndef __TEST_DATA_STRUCTURE_H__
#define __TEST_DATA_STRUCTURE_H__

#include "DataStructures.h"
#include "OldFiles/RootData.h"
#include "OldFiles/RootflyData.h"
#include "OldFiles/RootOverTime.h"
#include "OldFiles/Session_Array.h"
#include "stdafx.h"

namespace TestingData
{
  
  bool compareDataStructure(OldData::RootflyData oldStructure, Experiment *const curExp)
  {
    if(curExp->GetImageFormat()!=oldStructure.m_image_format)
    { 
      AfxMessageBox("Error! incorrect image Format");
      return false;
    }
    if(curExp->GetImageDir()!=oldStructure.m_image_directory)
    {
      AfxMessageBox("Error! incorrect image directory");
      return false;
    }
    if(curExp->GetMMConversion()!=oldStructure.m_mm_conversion)
    {
      AfxMessageBox("Error! incorrect MM Conversions");
      return false;
    }
    
    // variable declarations
    // possible root colors
    enum OldColor {COLOR_DARKBROWN,COLOR_LIGHTBROWN,COLOR_BLACK,COLOR_WHITE};
    int ntubes,ntransects,nwindows,nsessions;
    int t,w,p,tr, rb, s; //g
    CString temp_str;
    OldData::RootList *old_current_rootlist;
    int num_points, num_rootblocks;
    OldData::RootInstance *old_current_rootinstance;
    CPoint offset;
    OldColor root_color;
    bool status;
    // write number of tubes transect windows sessions
    oldStructure.m_root.GetDimensions(&ntubes,&ntransects,&nwindows,&nsessions);
    Tube *currentTube;
    Root *currentRoot;
    RootInstance *currentInstance;
    RDiameter *currentDiameter;
    RPoint *currentRootPoint;
    std::vector<CPoint> old_offsets;
    
    int rNum = 1;
    // loop through tubes t 
    for( t = 0; t < ntubes; t++ ) 
    {
      // loop through transects tr
      for( tr = 0; tr < ntransects; tr++ ) 
      {
        // check tube/transect format in new and compare with t and tr from old

        // Removed this line to match file name such as **_T001_***.jpg
        // Corey Dickerson (4/19/07)
        //temp_str.Format("%02d%c",t+1,65+tr);
        temp_str.Format("%03d",t+1);


        //AfxMessageBox("Tube: " + temp_str);
        currentTube = curExp->GetTube(temp_str);
        
        if(currentTube==NULL)
        {
          AfxMessageBox("Error! Tube NULL");
          return false;
        }
        //loop through windows w
        for( w = 0; w < nwindows; w++ ) 
        {   
          old_current_rootlist = oldStructure.m_root.GetRootList(t,tr,w);
          
          // Resize vector to be size of number of sessions
          // so there ARE holes
          old_offsets.resize(nsessions);
          
          // for each offset in all session
          for( int g = 0; g < nsessions; g++) 
          {
            // Get offset for this current session
            offset = old_current_rootlist->offset_list.at(g);
            
            //Put offset in new vector at corresponding location
            old_offsets.at(g) = offset;
          }

          // old_current_rootlist is the number of RootOverTime for a window 
          num_rootblocks = old_current_rootlist->size(); 
          
          // loop through root blocks rb
          for( rb = 0; rb < num_rootblocks; rb++ ) 
          {
            if(currentTube == NULL)
            {
              AfxMessageBox("Current Tube Null");
              return false;
            }
            currentRoot = currentTube->GetRoot(rNum++);
            
            OldData::RootOverTime& myrot = old_current_rootlist->at(rb);

            status = myrot.GetStatus();
            
            // loops through sessions for a rootblock s
            for (s = myrot.GetFirstSession() - 1 ; 
                 s < myrot.GetNumInstances() + myrot.GetFirstSession()-1 ; 
                 s++)
            {
              if(s >= old_offsets.size() )
                break;

              currentInstance = currentRoot->GetRootInstance(s+1);
              
              // num_points is the number of points in the current root
              old_current_rootinstance = old_current_rootlist->at(rb).GetRootInstance(s,&status);
              num_points = old_current_rootinstance->points.size();
              
              if(currentInstance->IsAlive() != status)
              {
                AfxMessageBox("Testing: Error in comparing status of root");
                return false;
              }
              
              // loop through points
              for( p = 0; p < num_points; p++ ) 
              {
                currentRootPoint = currentInstance->GetPoint(p);
                //check points **need to do**
                if(currentRootPoint->x != old_current_rootinstance->points.at(p).x + old_offsets.at(s).x)
                {
                  AfxMessageBox("Testing: Error! x value of point incorrect");
                  return false;
                }
                if(currentRootPoint->y != old_current_rootinstance->points.at(p).y + old_offsets.at(s).y)
                {
                  AfxMessageBox("Testing: Error! y value of point incorrect");
                  return false;
                }
                if(currentRootPoint->window != w+1)
                {
                  AfxMessageBox("Testing: Error! window of point incorrect");
                  return false;
                }
              } // end p
              
              //check diameter 
              currentDiameter = currentInstance->GetDiameter(0); 
              
              if(currentDiameter->radius != old_current_rootinstance->diameter / 2)
              {
                AfxMessageBox("Testing: Error! radius of diameter incorrect ");
                return false;
              }
              if(currentDiameter->r_point.x != (old_current_rootinstance->diameter_center.x))
              {
                
                AfxMessageBox("Testing: Error! x value of diameter incorrect");
                return false;
              }
              if(currentDiameter->r_point.y != (old_current_rootinstance->diameter_center.y))
              {
                AfxMessageBox("Testing: Error! y value of diameter incorrect");
                return false;
              }
              if(currentDiameter->r_point.window != w + 1)
              {
                AfxMessageBox("Testing: Error! window of diameter incorrect");
                return false;
              }
              //check color, need a GetColor function for the new data structure
              root_color = ((OldColor)old_current_rootinstance->color);
              if(!( (root_color == COLOR_DARKBROWN && currentInstance->GetColor() == COLOR_DARKBROWN) ||
                (root_color == COLOR_LIGHTBROWN && currentInstance->GetColor() == COLOR_LIGHTBROWN) ||
                (root_color == COLOR_WHITE && currentInstance->GetColor() ==  COLOR_WHITE ) ||
                (root_color == COLOR_BLACK && currentInstance->GetColor() == COLOR_BLACK )))
              {                  
                AfxMessageBox("Error! color is wrong");
                return false;
              }
              
            } // end s
          } // end r
        } // end w
      } // end tr
      rNum = 1;
    } // end t
    
    
    //AfxMessageBox("Testing: Comparing new data structure with old data structure-SUCCESS!");
    return true;
  }
  
}
#endif __TEST_DATA_STRUCTURE_H__